/**********************************************************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
 * other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
 * applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
 * EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
 * SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO
 * THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
 * this software. By using this software, you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 *
 * Copyright (C) 2021 Renesas Electronics Corporation. All rights reserved.
 *********************************************************************************************************************/
/******************************************************************************************************************//**
 * @file          r_bfe_api.h
 * @Version       2.0
 * @brief         Interface for communications with the BFE.
 *********************************************************************************************************************/
/**********************************************************************************************************************
 * History : DD.MM.YYYY Version  Description
 *         : 01.05.2021 1.00     First Release
 *         : 07.07.2022 2.00     API structure update
 *********************************************************************************************************************/

#ifndef BFE_R_BFE_API_H_
#define BFE_R_BFE_API_H_

/*******************************************************************************************************************//**
 * @ingroup RENESAS_INTERFACES
 * @defgroup BFE Interface
 * @brief Interface  for accessing and configuring Battery Front End.
 *
 * @section BFE_API_SUMMARY Summary
 * The Battery Front End shared interface provides the ability to access the resources and functionalities of the
 * Battery Front End.
 *
 * BFE Interface description: @ref BFE
 *
 * @{
 **********************************************************************************************************************/

/**********************************************************************************************************************
 Includes   <System Includes> , "Project Includes"
 *********************************************************************************************************************/
/* Generic headers */
#include <bfe/r_bfe_cfg.h>
#include <stdint.h>
#include <stdbool.h>

/**********************************************************************************************************************
 Macro definitions
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Global Typedef definitions
 *********************************************************************************************************************/

/** Common error codes */
typedef enum e_bfe_err
{
    BFE_SUCCESS                         = 0,    ///< BFE_SUCCESS

    BFE_ERR_ASSERTION                   = 1,    ///< Critical assertion has failed
    BFE_ERR_INVALID_POINTER             = 2,    ///< Pointer points to an invalid memory location
    BFE_ERR_INVALID_ARGUMENT            = 3,    ///< Invalid input parameter
    BFE_ERR_INVALID_NUMARGUMENTS        = 4,    ///< Invalid number of arguments
    BFE_ERR_INVALID_REGISTER            = 5,    ///< Register does not exist
    BFE_ERR_READ_ONLY_REGISTER          = 6,    ///< Register cannot be written
    BFE_ERR_INVALID_STATE               = 7,    ///< Selected state is invalid
    BFE_ERR_UNMATCHED_REGISTERS         = 8,    ///< MCU and device registers do not match
    BFE_ERR_UNSUPPORTED_FEATURE         = 9,    ///< Feature or function is not supported by the BFE
    BFE_ERR_DEVICE_BUSY                 = 10,   ///< Device is currently busy
    BFE_ERR_DEVICE_NOT_INITIALIZED      = 11,   ///< Device has not been initialized
    BFE_ERR_NOT_ENOUGH_MEMORY           = 12,   ///< Caller has not assigned enough memory
    BFE_ERR_REGISTER_RESET_UNMATCHED    = 13,   ///< Expected reset value does not match the actual register value
    BFE_ERR_INV_REG_SIZE                = 14,   ///< Register size is invalid
    BFE_ERR_UNSUPPORTED_MODE            = 15,   ///< Selected device mode is not supported
    BFE_ERR_INVALID_OPERATION           = 16,   ///< Operation not allowed
    BFE_ERR_INVALID_CELL                = 17,   ///< Invalid cell number
    BFE_ERR_WRITE_VERIFY                = 18,   ///< Write command verification error
    BFE_ERR_SCAN_CNTR                   = 19,   ///< Scan command was not received

    /* Start of communication specific. */
    BFE_ERR_COMM_TIMEOUT                = 21,   ///< Communication timeout exceeded
    BFE_ERR_MSG_BUF                     = 22,   ///< Message buffer overflow
    BFE_ERR_COMM_FAULT                  = 23,   ///< Communication error
    BFE_ERR_RESPONSE                    = 24,   ///< Response is invalid
    BFE_ERR_CRC_INCORRECT               = 25,   ///< Incorrect CRC in the data array
    BFE_ERR_COMM_UNSUP_INTERFACE        = 26,   ///< Selected communication interface is unsupported

    /* Start of device specific. */
    BFE_ERR_SLEEP                       = 30,   ///< Device sleep error
    BFE_ERR_WAKEUP                      = 31,   ///< Device wake up error
    BFE_ERR_EEPROM                      = 32,   ///< The EEPROM data is corrupted
    BFE_ERR_ADC                         = 33,   ///< ADC error
    BFE_ERR_MUX                         = 34,   ///< Multiplexer error
    BFE_ERR_CELL_BALANCE                = 35,   ///< Cell balancing error
    BFE_ERR_INVALID_DEV_ADDRESS         = 36,   ///< Invalid device address
    BFE_ERR_INVALID_CONF                = 37,   ///< Invalid BFE configuration
    BFE_ERR_MEAS_TIMEOUT                = 38,   ///< Measurement timeout exceeded
    BFE_ERR_FET_CONTROL                 = 39,   ///< Power FETs control error

    /* Start of daisy chain specific. */
    BFE_ERR_NA_STANDALONE               = 50,   ///< Not available in stand-alone operation
    BFE_ERR_DAISY_CHAIN_COMM            = 51,   ///< Error in daisy chain communication
    BFE_ERR_RESPONSE_TIMEOUT            = 52,   ///< Response is not received within the expected timeout
    BFE_ERR_ACK                         = 53,   ///< Acknowledge was not received
    BFE_ERR_NAK                         = 54,   ///< Not Acknowledge is received
    BFE_ERR_IDENTIFICATION              = 55,   ///< The stack identification has failed

    /* Start of FSP specific. */
    BFE_ERR_FSP                         = 61,   ///< Error in FSP layer

} e_bfe_err_t;

/** BFE types of resets */
typedef enum e_bfe_reset_type
{
    BFE_RESET_TYPE_SOFT = 0,                    ///< Reset only the digital part
    BFE_RESET_TYPE_HARD,                        ///< Reset both the digital and analog parts
    BFE_RESET_TYPE_TOIDLE,                      ///< Reset the device to idle state
} e_bfe_reset_type_t;

/** BFE process control options */
typedef enum e_bfe_process_ctrl
{
    BFE_PROCESS_ENABLE = 0,                     ///< Start a specific process
    BFE_PROCESS_INHIBIT,                        ///< Stop a specific process
} e_bfe_process_ctrl_t;

/** BFE power FETs driver configuration */
typedef enum e_bfe_driver_cfg
{
    BFE_DRIVER_HIGH_SIDE = 0,                   ///< FETs are disconnecting the positive terminal of the battery pack
    BFE_DRIVER_LOW_SIDE,                        ///< FETs are disconnecting the negative terminal of the battery pack
    BFE_DRIVER_NONE,                            ///< The BFE is not driving any power FETs
} e_bfe_driver_cfg_t;

/** BFE power FETs configuration */
typedef enum e_bfe_fet_cfg
{
    BFE_FET_CONFIG_SERIES = 0,                  ///< Battery pack has one set of terminals for charge and discharge
    BFE_FET_CONFIG_PARALLEL,                    ///< Battery pack has separate terminals for charge and discharge
    BFE_FET_NONE,                               ///< The BFE is not driving any power FETs
} e_bfe_fet_cfg_t;

/** BFE Discharge FETs state options */
typedef enum e_bfe_dfet_state
{
    BFE_DFET_ON = 0,                            ///< The Discharge FET is conducting
    BFE_DFET_OFF,                               ///< The Discharge FET is not conducting
} e_bfe_dfet_state_t;

/** BFE Charge FETs state options */
typedef enum e_bfe_cfet_state
{
    BFE_CFET_ON = 0,                            ///< The Charge FET is conducting
    BFE_CFET_OFF,                               ///< The Charge FET is not conducting
} e_bfe_cfet_state_t;

/** BFE communication interface options */
typedef enum e_bfe_comm_interface
{
    BFE_COMMUNICATION_INTERFACE_SPI = 0,        ///< Serial Peripheral Interface
    BFE_COMMUNICATION_INTERFACE_SCI,            ///< Serial Communication Interface
    BFE_COMMUNICATION_INTERFACE_I2C,            ///< Inter-Integrated Circuit communication interface
    BFE_COMMUNICATION_SINGLE_WIRE,              ///< Single wire communication interface
} e_bfe_comm_interface_t;

/** BFE states and modes */
typedef enum e_bfe_mode
{
    BFE_STATE_RESET = 0,                        ///> Initial state when all circuits and oscillators are off
    BFE_STATE_POWER_UP,                         ///> BFE power-up state
    BFE_MODE_IDLE,                              ///> The device is ready waiting for a command to be executed
    BFE_STATE_SCAN,                             ///> Single execution of measurements
    BFE_MODE_AUTOSCAN,                          ///> Periodic execution of measurements
    BFE_MODE_LOW_POWER,                         ///> BFE low power mode
    BFE_MODE_SHIP,                              ///> BFE with lowest power consumption
    BFE_MODE_POWER_DOWN,                        ///> BFE shut down mode
} e_bfe_mode_t;

/** BFE diagnostic options enumeration */
typedef enum e_bfe_diag_option
{
    BFE_FULL_TEST = 0,                          ///> Run a complete self-test
    BFE_TEST_ADC,                               ///> Test the ADC
    BFE_TEST_MUX,                               ///> Test the multiplexer
    BFE_TEST_CB,                                ///> Test cell balancing circuit
    BFE_TEST_OW,                                ///> Check for open wires
    BFE_TEST_CUSTOM                             ///> Device specific test option
} e_bfe_diag_option_t;

/** BFE memory check options enumeration */
typedef enum e_bfe_mem_check_option
{
    BFE_CHECK_CONF_REGS = 0,                    ///> Verify content of configuration registers
    BFE_CHECK_EEPROM,                           ///> Verify content of EEPROM memory
    BFE_CHECK_DEF_VALS,                         ///> Check registers for default values
    BFE_CHECK_CUSTOM                            ///> Device specific memory check option
} e_bfe_mem_check_option_t;

/** BFE battery pack current measurement structure */
typedef struct st_bfe_i_pack_meas
{
    float                         i_charge;         ///< Charge current [A]
    float                         i_discharge;      ///< Discharge current [A]
} st_bfe_i_pack_meas_t;

/** BFE interface configuration */
typedef struct st_bfe_cfg
{
    const uint16_t              * p_cells_select;   ///< Pointer to a constant for existing cells in the battery pack
    const uint16_t              * p_temps_select;   ///< Pointer to a constant for used temp inputs in the BFE
    const e_bfe_comm_interface_t  peripheral_type;  ///< Communication interface used between the BFE and the MCU
    const e_bfe_driver_cfg_t      driver_cfg;       ///< Configuration of the FET driver
    const e_bfe_fet_cfg_t         fet_cfg;          ///< Configuration of power FETs to manage charge and discharge
    void const                  * p_extend;         ///< Pointer to extended BFE configuration structure
} st_bfe_cfg_t;

/** BFE control structure */
typedef struct st_bfe_ctrl
{
    bool                          is_initialized;       ///< Indicates if the BFE is initialized
    bool                          is_low_power;         ///< Indicates if the BFE is in low power mode
    bool                          is_balancing;         ///< Indicates if the BFE is balancing cells
    bool                          is_cont_scanning;     ///< Indicates if the BFE is continuously scanning
    bool                          is_fault_detected;    ///< Indicates if the BFE has detected a fault
    void                        * p_bfe_info;           ///< Pointer to the BFE information structure
    void                        * p_bfe_regs;           ///< Pointer to the BFE registers
    const st_bfe_cfg_t          * p_cfg;                ///< Pointer to configuration entity
    void                        * p_extend;             ///< Pointer to extended BFE control structure
} st_bfe_ctrl_t;

/** Device register container
 * @par Implemented as
 * - st_raa489204_quick_reg_t
 * - st_raa489220_quick_reg_t
 **/
typedef void bfe_register_t;

/** Cells voltage measurement data structure
 * @par Implemented as
 * - u_raa489204_voltages_meas_t
 * - u_raa489220_vcells_meas_t
 **/
typedef void bfe_vcell_meas_t;

/** Temperatures measurement data structure
 * @par Implemented as
 * - u_raa489204_temps_meas_t
 * - u_raa489220_temps_meas_t
 **/
typedef void bfe_temp_meas_t;

/** All inputs measurement data structure
 * @par Implemented as
 * - u_raa489204_all_meas_t
 * - u_raa489220_all_meas_t
 **/
typedef void bfe_all_meas_t;

/** Other measurements data structure
 * @par Implemented as
 * - u_raa489204_mixed_voltages_meas_t
 * - u_raa489220_sys_scan_meas_t
 **/
typedef void bfe_other_meas_t;

/** BFE faults structure
 * @par Implemented as
 * - st_raa489204_faults_t
 * - st_raa489220_faults_t
 **/
typedef void bfe_faults_t;

/** Watchdog timer control structure
 * @par Implemented as
 * - raa489204_watchdog_ctrl_t
 **/
typedef void bfe_watchdog_ctrl_t;

/** Cell balancing configuration structure
 * @par Implemented as
 * - raa489204_cb_cfg_t
 **/
typedef void bfe_cb_cfg_t;

/** Continuous scan configuration structure
 * @par Implemented as
 * - raa489204_scan_cont_cfg_t
 **/
typedef void bfe_scan_cont_cfg_t;

/** GPIO control structure
 * @par Implemented as
 * - st_raa489204_gpio_ctrl_t
 * - st_raa489220_hvgpio_ctrl_t
 **/
typedef void bfe_gpio_ctrl_t;

/** Extend API Functions
 * @par Implemented as
 * -
 **/
typedef void * extendApi_t;

/** Shared interface definition for BFE */
typedef struct st_bfe_api
{
    /** Initializes a BFE interface by enabling and configuring the necessary peripheral modules of the MCU, identifying
     *  the BFE and other device-specific actions.
     * @par Implemented as
     * - @ref R_RAA489204_Init()
     * - @ref R_RAA489220_Init()
     * @param[in, out]  p_ctrl  Pointer to the control structure
     * @param[in]       p_cfg   Pointer to the configuration structure
     */
    e_bfe_err_t ( * p_initialize) (st_bfe_ctrl_t * const p_ctrl, st_bfe_cfg_t const * const p_cfg);

    /** Deinitializes a BFE. Disables the used peripheral modules of the MCU.
     * @par Implemented as
     * - @ref R_RAA489204_Deinit()
     * - @ref R_RAA489220_Deinit()
     * @param[in, out]  p_ctrl  Pointer to the control structure
     */
    e_bfe_err_t ( * p_deinitialize) (st_bfe_ctrl_t * const p_ctrl);

    /** Configures a BFE device or stack of devices by writing into all configuration registers.
     * @par Implemented as
     * - @ref R_RAA489204_Setup()
     * - @ref R_RAA489220_Setup()
     * @param[in, out]  p_ctrl  Pointer to the control structure
     * @param[in]       p_cfg   Pointer to the configuration structure
     */
    e_bfe_err_t ( * p_setup) (st_bfe_ctrl_t * const p_ctrl, st_bfe_cfg_t const * const p_cfg);

    /** Resets a BFE.
     * @par Implemented as
     * - @ref R_RAA489204_Reset()
     * - @ref R_RAA489220_Reset()
     * @param[in, out]  p_ctrl  Pointer to the control structure
     * @param[in]       type    Type of reset
     */
    e_bfe_err_t ( * p_reset) (st_bfe_ctrl_t * const p_ctrl, e_bfe_reset_type_t type);

    /** Forces the BFE to enter a mode or a state.
     * @par Implemented as
     * - @ref R_RAA489204_ModeSet()
     * - @ref R_RAA489220_ModeSet()
     * @param[in, out]  p_ctrl  Pointer to the control structure
     * @param[in]       mode    Mode to be set
     */
    e_bfe_err_t ( * p_modeSet) (st_bfe_ctrl_t * const p_ctrl, e_bfe_mode_t mode);

    /** Reads the current mode of a BFE.
     * @par Implemented as
     * - @ref R_RAA489204_ModeRead()
     * - @ref R_RAA489220_ModeRead()
     * @param[in, out]  p_ctrl  Pointer to the control structure
     * @param[in]       p_mode  Pointer to the obtained mode
     */
    e_bfe_err_t ( * p_modeRead) (st_bfe_ctrl_t * const p_ctrl, e_bfe_mode_t * const p_mode);

    /** Tests communication between a MCU and single or multiple BFE devices.
     * @par Implemented as
     * - @ref R_RAA489204_CommTest()
     * - @ref R_RAA489220_CommTest()
     * @param[in, out]  p_ctrl  Pointer to the control structure
     */
    e_bfe_err_t ( * p_commTest) (st_bfe_ctrl_t * const p_ctrl);

    /** Runs a self-diagnostic tests for a BFE.
     * @par Implemented as
     * - @ref R_RAA489204_SelfDiag()
     * - @ref R_RAA489220_SelfDiag()
     * @param[in, out]  p_ctrl  Pointer to the control structure
     * @param[in]       option  Selected option for the self-diagnostic
     */
    e_bfe_err_t ( * p_selfDiagnostic) (st_bfe_ctrl_t * const p_ctrl, e_bfe_diag_option_t option);

    /** Tests BFE memory for corrupted registers or data.
     * @par Implemented as
     * - @ref R_RAA489204_MemCheck()
     * - @ref R_RAA489220_MemCheck()
     * @param[in, out]  p_ctrl  Pointer to the control structure
     * @param[in]       option  Selected option for the memory test
     */
    e_bfe_err_t ( * p_memoryCheck) (st_bfe_ctrl_t * const p_ctrl, e_bfe_mem_check_option_t option);

    /** Acquires a battery pack voltage with a BFE. The return data is converted into voltage.
     * @par Implemented as
     * - @ref R_RAA489204_VPackGet()
     * - @ref R_RAA489220_VPackGet()
     * @param[in, out]  p_ctrl  Pointer to the control structure
     * @param[out]      p_value Pointer to the acquired data
     * @param[in]       trigger Specify if a measurement must be triggered or only read data for the last one.
     */
    e_bfe_err_t ( * p_vPackGet) (st_bfe_ctrl_t * const p_ctrl,
                                 float         * const p_value,
                                 bool                  trigger);

    /** Acquires a battery pack current with a BFE. The return data is converted into current.
     * @par Implemented as
     * - @ref R_RAA489204_IPackGet()
     * - @ref R_RAA489220_IPackGet()
     * @param[in, out]  p_ctrl  Pointer to the control structure
     * @param[out]      p_value Pointer to the acquired data
     * @param[in]       trigger Specify if a measurement must be triggered or only read data for the last one.
     */
    e_bfe_err_t ( * p_iPackGet) (st_bfe_ctrl_t        * const p_ctrl,
                                 st_bfe_i_pack_meas_t * const p_values,
                                 bool                         trigger);

    /** Acquires the voltages of all cells in the battery pack. The return data is converted into voltage.
     * @par Implemented as
     * - @ref R_RAA489204_VCellsGet()
     * - @ref R_RAA489220_VCellsGet()
     * @param[in, out]  p_ctrl   Pointer to the control structure
     * @param[out]      p_values Pointer to the acquired data
     * @param[in]       trigger  Specify if a measurement must be triggered or only read data for the last one.
     */
    e_bfe_err_t ( * p_vCellsGet) (st_bfe_ctrl_t    * const p_ctrl,
                                  bfe_vcell_meas_t * const p_values,
                                  bool                     trigger);

    /** Acquires temperatures with a BFE. The return data is converted into voltage.
     * @par Implemented as
     * - @ref R_RAA489204_Temps()
     * - @ref R_RAA489220_Temps()
     * @param[in, out]  p_ctrl   Pointer to the control structure
     * @param[out]      p_values Pointer to acquired data
     * @param[in]       trigger  Specify if a measurement must be triggered or only read data for the last one.
     */
    e_bfe_err_t ( * p_temperaturesGet) (st_bfe_ctrl_t   * const p_ctrl,
                                        bfe_temp_meas_t * const p_values,
                                        bool                    trigger);

    /** Acquires all measurable parameters with a BFE. The return data is converted into the relevant units.
     * @par Implemented as
     * - @ref R_RAA489204_AllGet()
     * - @ref R_RAA489220_AllGet()
     * @param[in, out]  p_ctrl   Pointer to the control structure
     * @param[out]      p_values Pointer to acquired data
     * @param[in]       trigger  Specify if a measurement must be triggered or only read data for the last one.
     */
    e_bfe_err_t ( * p_allGet) (st_bfe_ctrl_t  * const p_ctrl,
                               bfe_all_meas_t * const p_values,
                               bool                   trigger);

    /** Acquires custom parameters with a BFE. The return data is converted into the relevant units.
     * @par Implemented as
     * - @ref R_RAA489204_VMixGet()
     * - @ref R_RAA489220_SysScanGet()
     * @param[in, out]  p_ctrl   Pointer to the control structure
     * @param[out]      p_values Pointer to acquired data
     * @param[in]       trigger  Specify if a measurement must be triggered or only read data for the last one.
     */
    e_bfe_err_t ( * p_otherGet) (st_bfe_ctrl_t    * const p_ctrl,
                                 bfe_other_meas_t * const p_values,
                                 bool                     trigger);

    /** Reads all fault registers of a BFE.
     * @par Implemented as
     * - @ref R_RAA489204_FaultsAllRead()
     * - @ref R_RAA489220_FaultsAllRead()
     * @param[in, out]  p_ctrl   Pointer to the control structure
     * @param[out]      p_faults Pointer to faults data structure
     */
    e_bfe_err_t ( * p_faultsAllRead) (st_bfe_ctrl_t * const p_ctrl, bfe_faults_t * const p_faults);

    /** Checks a BFE for faults.
     * @par Implemented as
     * - @ref R_RAA489204_FaultsCheck()
     * - @ref R_RAA489220_FaultsCheck()
     * @param[in, out]  p_ctrl  Pointer to the control structure
     */
    e_bfe_err_t ( * p_faultsCheck) (st_bfe_ctrl_t * const p_ctrl);

    /** Attempts to clear all faults in a BFE.
     * @par Implemented as
     * - @ref R_RAA489204_FaultsAllClear()
     * - @ref R_RAA489220_FaultsAllClear()
     * @param[in, out]  p_ctrl  Pointer to the control structure
     * @param[out]      p_fault Pointer to boolean faults clear status variable
     */
    e_bfe_err_t ( * p_faultsAllClear) (st_bfe_ctrl_t * const p_ctrl, bool * const p_success);

    /** Configures and controls cell balancing in a BFE.
     * @par Implemented as
     * - @ref R_RAA489204_CellBalanceCtrl()
     * - @ref R_RAA489220_CellBalanceCtrl()
     * @param[in, out]  p_ctrl      Pointer to the control structure
     * @param[in]       p_bal_cfg   Pointer to the balancing configuration structure
     * @param[in]       ctrl_option Specify action to enable or inhibit cell balancing.
     */
    e_bfe_err_t ( * p_cellBalanceControl) (st_bfe_ctrl_t       * const p_ctrl,
                                           bfe_cb_cfg_t        * const p_bal_cfg,
                                           e_bfe_process_ctrl_t        ctrl_option);

    /** Checks if cell balancing is in progress.
     * @par Implemented as
     * - @ref R_RAA489204_IsCellBalancing()
     * - @ref R_RAA489220_IsCellBalancing()
     * @param[in, out]  p_ctrl   Pointer to the control structure
     */
    e_bfe_err_t ( * p_isCellBalancing) (st_bfe_ctrl_t * const p_ctrl);

    /** Controls scan continuous function of a BFE.
     * @par Implemented as
     * - @ref R_RAA489204_ContScanCtrl()
     * - @ref R_RAA489220_ContScanCtrl()
     * @param[in, out]  p_ctrl      Pointer to the control structure
     * @param[in]       p_bal_cfg   Pointer to the continuous scan configuration structure
     * @param[in]       ctrl_option Specify action to enable or inhibit continuous scan.
     */
    e_bfe_err_t ( * p_continuousScanControl) (st_bfe_ctrl_t       * const p_ctrl,
                                              bfe_scan_cont_cfg_t * const p_scan_cfg,
                                              e_bfe_process_ctrl_t        ctrl_option);

    /** Controls watchdog timer function in a BFE.
     * @par Implemented as
     * - @ref R_RAA489204_WatchdogCtrl()
     * - @ref R_RAA489220_WatchdogCtrl()
     * @param[in, out]  p_ctrl    Pointer to the control structure
     * @param[in]       p_options Pointer to the watchdog timer control options
     */
    e_bfe_err_t ( * p_wdControl) (st_bfe_ctrl_t * const p_ctrl, bfe_watchdog_ctrl_t * const p_options);

    /** Controls power FETs in a BFE.
     * @par Implemented as
     * - @ref R_RAA489204_FETsCtrl()
     * - @ref R_RAA489220_FETsCtrl()
     * @param[in, out]  p_ctrl      Pointer to the control structure
     * @param[in]       group_num   Select FET group to control.
     * @param[in]       c_fet_state Specify state of the charge FET control pin.
     * @param[in]       d_fet_state Specify state of the discharge FET control pin.
     */
    e_bfe_err_t ( * p_fetControl) (st_bfe_ctrl_t   * const p_ctrl,
                                   uint8_t                 group_num,
                                   e_bfe_cfet_state_t      c_fet_state,
                                   e_bfe_dfet_state_t      d_fet_state);

    /** Controls GPIO pins in a BFE.
     * @par Implemented as
     * - @ref R_RAA489204_GPIOsCtrl()
     * - @ref R_RAA489220_HVGPIOCtrl()
     * @param[in, out]  p_ctrl      Pointer to the control structure
     * @param[in]       p_options   Pointer to GPIO pins control structure.
     */
    e_bfe_err_t ( * p_gpioControl) (st_bfe_ctrl_t * const p_ctrl, bfe_gpio_ctrl_t * const p_options);

    /** Reads a register in a BFE.
     * @par Implemented as
     * - @ref R_RAA489204_RegisterRead()
     * - @ref R_RAA489220_RegisterRead()
     *
     * @param[in]  p_ctrl     Pointer to the BFE control block.
     * @param[in]  p_register Pointer to the target BFE register container.
     *
     * @return bfe_err error code
     */
    e_bfe_err_t ( * p_registerRead) (st_bfe_ctrl_t * const p_ctrl, bfe_register_t * const p_register);

    /** Writes in a register of a BFE.
     * @par Implemented as
     * - @ref R_RAA489204_RegisterWrite()
     * - @ref R_RAA489220_RegisterWrite()
     *
     * @param[in]  p_ctrl     Pointer to the BFE control block.
     * @param[in]  p_register Pointer to the target BFE register container.
     *
     * @return bfe_err error code
     */
    e_bfe_err_t ( * p_registerWrite) (st_bfe_ctrl_t * const p_ctrl, bfe_register_t * const p_register);

    /** Extend the API functions defined above. It might be a structure containing pointers to user-defined functions.
     */
    extendApi_t  p_extendApiFunctions;

} bfe_api_t;

/** This structure comprises everything that is needed to use an instance of this interface. */
typedef struct st_bfe_instance
{
    st_bfe_ctrl_t       * p_ctrl;     ///< Pointer to the control structure for this instance
    st_bfe_cfg_t  const * p_cfg;      ///< Pointer to the configuration structure for this instance
    bfe_api_t     const * p_api;      ///< Pointer to the API structure for this instance
} bfe_instance_t;

/**********************************************************************************************************************
 External global variables
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Exported global functions
 *********************************************************************************************************************/

#endif /* BFE_R_BFE_API_H_ */

/*******************************************************************************************************************//**
 * @} (end defgroup IOPORT_API)
 **********************************************************************************************************************/
