/**********************************************************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
 * other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
 * applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
 * EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
 * SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO
 * THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
 * this software. By using this software, you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 *
 * Copyright (C) 2021 Renesas Electronics Corporation. All rights reserved.
 *********************************************************************************************************************/
/******************************************************************************************************************//**
 * @file          r_raa489220.h
 * @Version       1.00
 * @brief         Contains macros, data structures and declarations of functions used in r_raa489220.c
 *********************************************************************************************************************/
/**********************************************************************************************************************
 * History : DD.MM.YYYY Version  Description
 *         : 01.06.2022 1.00     First Release
 *********************************************************************************************************************/

/*******************************************************************************************************************//**
 * @addtogroup RAA489220
 * @{
 **********************************************************************************************************************/

#ifndef BFE_R_RAA489220_H_
#define BFE_R_RAA489220_H_

/**********************************************************************************************************************
 Includes   <System Includes> , "Project Includes"
 *********************************************************************************************************************/
/* Generic headers */
#include <bfe/r_bfe_api.h>
#include <bfe/r_bfe_common.h>
#include <stdint.h>
#include <stdbool.h>

/* BFE related headers */
#include "hal_data.h"

/**********************************************************************************************************************
 Macro definitions
 *********************************************************************************************************************/
#define BFE_CELL_NUM                    (10U)       ///< Number of cells
#define BFE_THERM_NUM                   (2U)        ///< Number of thermistors

/* I2C communication */
#define BFE_I2C_ADDRESS_CRC             (0x56)      ///< I2C address for CRC package
#define BFE_I2C_ADDRESS_NO_CRC          (0x12)      ///< I2C address for non CRC package

#define BFE_I2C_DATA_LGTH               (18U)       ///< I2C data buffer length

#define BFE_MAX_READ_DATA_LGTH          (7U)        ///< Maximum read data size

#define BFE_I2C_TRANSMIT_BYTES          (3U)        ///< I2C write package size [Bytes] (without slave address)
#define BFE_I2C_RECEIVE_BYTES           (2U)        ///< I2C read package size [Bytes]
#define BFE_I2C_RECEIVE_CMND_FS_BYTES   (4U)        ///< I2C faults and status bulk read package size [Bytes]
#define BFE_I2C_RECEIVE_CMND_RM_BYTES   (14U)       ///< I2C read measurements bulk read package size [Bytes]
#define BFE_I2C_RECEIVE_CMND_IP_BYTES   (4U)        ///< I2C pack current bulk read package size [Bytes]
#define BFE_I2C_ADDR_SIZE_BYTES         (1U)        ///< I2C register address size [Bytes]
#define BFE_I2C_ADDR_HEADER_SIZE_BYTES  (3U)        ///< I2C slave and register addresses container size [Bytes]
#define BFE_I2C_CRC_SIZE_BYTES          (1U)        ///< I2C CRC container size [Bytes]

/* RAA48220 voltage limits */
#define BFE_VCELL_OVLO_TH_MIN_V         (3.00f)     ///< Vcell Overvoltage lockout threshold minimum value [V]
#define BFE_VCELL_OVLO_TH_MAX_V         (4.50f)     ///< Vcell Overvoltage lockout threshold maximum value [V]
#define BFE_VCELL_OVLO_TH_STEP_V        (0.10f)     ///< Vcell Overvoltage lockout threshold step [V]
#define BFE_VCELL_OV_TH_MIN_V           (3.23f)     ///< Vcell Overvoltage threshold minimum value [V]
#define BFE_VCELL_OV_TH_MAX_V           (4.50f)     ///< Vcell Overvoltage threshold maximum value [V]
#define BFE_VCELL_OV_TH_STEP_V          (0.01f)     ///< Vcell Overvoltage threshold step [V]
#define BFE_VCELL_UV_TH_MIN_V           (1.50f)     ///< Vcell Undervoltage threshold minimum value [V]
#define BFE_VCELL_UV_TH_MAX_V           (3.00f)     ///< Vcell Undervoltage threshold maximum value [V]
#define BFE_VCELL_UV_TH_STEP_V          (0.10f)     ///< Vcell Undervoltage threshold step [V]
#define BFE_VCELL_UVLO_TH_MIN_V         (1.50f)     ///< Vcell Undervoltage lockout threshold minimum value [V]
#define BFE_VCELL_UVLO_TH_MAX_V         (3.00f)     ///< Vcell Undervoltage lockout threshold maximum value [V]

#define BFE_DVCELL_OV_TH_MIN_V          (0.50f)     ///< Delta Vcell Overvoltage threshold minimum value [V]
#define BFE_DVCELL_OV_TH_MAX_V          (2.00f)     ///< Delta Vcell Overvoltage threshold maximum value [V]
#define BFE_DVCELL_OV_TH_STEP_V         (0.10f)     ///< Delta Vcell Overvoltage threshold step [V]

#define BFE_VCS_MIN_V                   (0.001f)    ///< Current Sense Voltage minimum value [V]
#define BFE_VCS_MAX_V                   (0.051f)    ///< Current Sense Voltage maximum value [V]
#define BFE_VCS_STEP_V                  (0.001f)    ///< Current Sense Voltage step [V]

#define BFE_DOC_DISABLE                 (0x33)      ///< Disable discharge overcurrent protection constant

/* RAA48220 timings */
#define BFE_BUSY_MAX_CNT                (0xFFFF)    ///< Max wait count for Busy bit clear
#define BFE_BUSY_MIN_CNT                (0xFE69)    ///< Min wait count for Busy bit clear (~150 ms)
#define BFE_BUSY_BIT_READ_TIMEOUT_US    (1000U)     ///< Busy bit read timeout [us]
#define BFE_START_UP_DELAY_MS           (100U)      ///< Start-up delay (power on or enable) [ms]
#define BFE_THERM2_RST_HOLD_TIME_MS     (550U)      ///< THERM2 pin hold time to trigger a reset [ms]

/* Configuration registers 0x80-0x84 unlock codes */
#define BFE_USER_SELECT_LOCK_KEY_0      (0x0000)
#define BFE_USER_SELECT_UNLOCK_KEY_1    (0x6F9C)
#define BFE_USER_SELECT_UNLOCK_KEY_2    (0x70A5)
#define BFE_USER_SELECT_UNLOCK_KEY_3    (0x48E1)

/* Register defaults */
#define BFE_PACK_VOLTAGE_DEF            (0x0000)    ///< Pack Voltage register default
#define BFE_CELL_MAX_VOLTAGE_DEF        (0x0000)    ///< Cell Max Voltage register default
#define BFE_CELL_MIN_VOLTAGE_DEF        (0x0000)    ///< Cell Min Voltage register default
#define BFE_THERMISTOR_1_DEF            (0x0000)    ///< Thermistor 1 register default
#define BFE_THERMISTOR_2_DEF            (0x0000)    ///< Thermistor 2 register default
#define BFE_DISCHARGE_CURRENT_DEF       (0x0000)    ///< Discharge Current register default
#define BFE_CHARGE_CURRENT_DEF          (0x0000)    ///< Charge Current register default
#define BFE_FAULTS_DEF                  (0x0000)    ///< Faults register default
#define BFE_MASKS_AND_STATUS_DEF        (0xFE00)    ///< Masks & Status register default
#define BFE_SYSTEM_CONFIG_1_DEF         (0x0800)    ///< System Config 1 register default
#define BFE_MEASURE_SELECT_DEF          (0x0000)    ///< Measure Select register default
#define BFE_SYSTEM_CONFIG_2_DEF         (0x4000)    ///< System Config 2 register default
#define BFE_SYSTEM_CONFIG_3_DEF         (0xEA80)    ///< System Config 3 register default
#define BFE_OV_AND_EOC_DEF              (0xCC2D)    ///< OV and EOC register default
#define BFE_DV_CELL_OV_AND_UV_DEF       (0x0D25)    ///< DVcell OV, UV Thresholds register default
#define BFE_SCC_OT_AND_UT_DEF           (0x98B4)    ///< SCC, OT/UT Thresholds register default
#define BFE_CURRENT_DEF                 (0xCC71)    ///< Current Thresholds register default
#define BFE_DIE_REVISION_DEF            (0x0000)    ///< Die Revision register default
#define BFE_MANUFACTURING_ID_DEF        (0x0049)    ///< Manufacturing ID register default
#define BFE_DEVICE_ID_DEF               (0x0010)    ///< Device ID register default
#define BFE_USER_SELECT_MODE_DEF        (0x0000)    ///< User Select Mode register default
#define BFE_FAULT_AND_STATUS_DEF        (0x0000)    ///< Faults and Status command default
#define BFE_READ_MEASUREMENTS_DEF       (0x0000)    ///< Read Measurements command default
#define BFE_I_PACK_DEF                  (0x0000)    ///< Pack Current command default

/**********************************************************************************************************************
 Global Typedef definitions
 *********************************************************************************************************************/

/** Battery Front End registers full addresses */
/* Please, refer to Register Map table from RAA489220 Datasheet! */
typedef enum e_raa489220_reg_addr
{
    /* Measurement Results */
    BFE_PACK_VOLTAGE        = 0x00,                 ///< Pack Voltage register address
    BFE_CELL_MAX_VOLTAGE    = 0x01,                 ///< Cell Max Voltage register address
    BFE_CELL_MIN_VOLTAGE    = 0x02,                 ///< Cell Min Voltage register address
    BFE_THERMISTOR_1        = 0x03,                 ///< Thermistor 1 register address
    BFE_THERMISTOR_2        = 0x04,                 ///< Thermistor 2 register address
    BFE_DISCHARGE_CURRENT   = 0x05,                 ///< Discharge Current register address
    BFE_CHARGE_CURRENT      = 0x06,                 ///< Charge Current register address

    /* Faults and Status */
    BFE_FAULTS              = 0x10,                 ///< Faults register address
    BFE_MASKS_AND_STATUS    = 0x11,                 ///< Masks & Status register address

    /* Set Up */
    BFE_SYSTEM_CONFIG_1     = 0x40,                 ///< System Configuration 1 register address
    BFE_MEASURE_SELECT      = 0x41,                 ///< Measure Select register address
    BFE_SYSTEM_CONFIG_2     = 0x42,                 ///< System Configuration 2 register address

    BFE_USER_SELECT_MODE    = 0x6F, // Configuration registers 0x80-0x84 access

    /* Thresholds */
    BFE_SYSTEM_CONFIG_3     = 0x80,                 ///< System Configuration 3 register address
    BFE_OV_AND_EOC          = 0x81,                 ///< OV and EOC register address
    BFE_DV_CELL_OV_AND_UV   = 0x82,                 ///< DVcell OV, UV register address
    BFE_SCC_OT_AND_UT       = 0x83,                 ///< SCC, OT/UT register address
    BFE_CURRENT             = 0x84,                 ///< Current Thresholds register address

    /* Commands */
    BFE_FAULT_AND_STATUS    = 0xC1,                 ///< Faults and Status command address
    BFE_READ_MEASUREMENTS   = 0xC2,                 ///< Read Measurements command address
    BFE_I_PACK              = 0xC3,                 ///< Pack Current command address

    /* Product Information */
    BFE_DIE_REVISION        = 0xFD,                 ///< Die Revision register address
    BFE_MANUFACTURING_ID    = 0xFE,                 ///< Manufacturing ID register address
    BFE_DEVICE_ID           = 0xFF,                 ///< Device ID register address
} e_raa489220_reg_addr_t;

/** RAA489220 register type */
typedef enum e_raa489220_reg_type
{
    READ_ONLY = 0,                                  ///< Read only register
    READ_WRITE,                                     ///< Read/Write register
    COMMAND,                                        ///< Command
} e_raa489220_reg_type_t;

/** RAA489220 register container */
typedef struct
{
    const e_raa489220_reg_addr_t  address;          ///< Register Address
    const e_raa489220_reg_type_t  type;             ///< Register type
    const uint16_t                def_value;        ///< Register default value
    uint16_t                      value;            ///< Register last value
} st_raa489220_register_t;

/** RAA489220 quick register access structure. */
typedef struct st_raa489220_quick_reg
{
    st_raa489220_register_t   * p_targer_register;  ///< Pointer to register container
    uint16_t                    data;               ///< Data field
}
st_raa489220_quick_reg_t;

/** RAA489220 registers entity */
typedef struct st_raa489220_registers
{
    st_raa489220_register_t pack_voltage;           ///< Pack voltage - Read only
    st_raa489220_register_t cell_max_voltage;       ///< Maximum cell voltage - Read only
    st_raa489220_register_t cell_min_voltage;       ///< Minimum cell voltage - Read only
    st_raa489220_register_t thermistor_1;           ///< Thermistor 1 voltage - Read only
    st_raa489220_register_t thermistor_2;           ///< Thermistor 2 voltage - Read only
    st_raa489220_register_t discharge_current;      ///< Discharge current - Read only
    st_raa489220_register_t charge_current;         ///< Charge current - Read only
    st_raa489220_register_t faults;                 ///< Faults - Read only
    st_raa489220_register_t masks_status;           ///< Masks & status - Read/Write
    st_raa489220_register_t system_config_1;        ///< System configuration 1 - Read/Write
    st_raa489220_register_t measure_select;         ///< Measure select - Read/Write
    st_raa489220_register_t system_config_2;        ///< System configuration 2 - Read/Write
    st_raa489220_register_t system_config_3;        ///< System configuration 3 - Read/Write
    st_raa489220_register_t ov_eoc;                 ///< Overvoltage & end of charge - Read/Write
    st_raa489220_register_t dvcov_uv;               ///< Delta Vcell overvoltage & undervoltage - Read/Write
    st_raa489220_register_t scc_ot_ut;              ///< Short circuit delay & over/under temperature - Read/Write
    st_raa489220_register_t current;                ///< Current thresholds - Read/Write
    st_raa489220_register_t die_revision;           ///< Die revision - Read only
    st_raa489220_register_t manufacturing_id;       ///< Manufacturing ID - Read only
    st_raa489220_register_t device_id;              ///< Device ID - Read only
    st_raa489220_register_t user_sel_mode;          ///< User select mode - Read/Write
    st_raa489220_register_t cmnd_fault_stat;        ///< Read fault and status - Command
    st_raa489220_register_t cmnd_measurements;      ///< Read measurements - Command
    st_raa489220_register_t cmnd_i_pack;            ///< Read charge and discharge current - Command
} st_raa489220_registers_t;

/** System registers */
/* 0x10 Faults Register */
typedef union u_raa489220_fault_reg
{
    uint16_t value;

    struct
    {
        uint16_t COCF               : 1;    // 0x10.0  Charge Overcurrent fault - RO
        uint16_t DOCF               : 1;    // 0x10.1  Discharge Overcurrent fault - RO
        uint16_t SCF                : 1;    // 0x10.2  Short-Circuit fault - RO
        uint16_t UTF                : 1;    // 0x10.3  Under-Temperature fault - RO
        uint16_t OTF                : 1;    // 0x10.4  Over-Temperature fault - RO
        uint16_t UVF                : 1;    // 0x10.5  Undervoltage fault - RO
        uint16_t OVF                : 1;    // 0x10.6  Overvoltage fault - RO
        uint16_t DCVF               : 1;    // 0x10.7  Delta Cell Voltage fault - RO
        uint16_t COMMTO             : 1;    // 0x10.8  Communication Time Out - RO
        uint16_t RSV_9              : 1;    // 0x10.9  Reserved - RO
        uint16_t STF                : 1;    // 0x10.10 Self Test Fault - RO
        uint16_t LOF                : 1;    // 0x10.11 Lock out fault - RO
        uint16_t VTMPF              : 1;    // 0x10.12 VTEMP fault - RO
        uint16_t IOTF               : 1;    // 0x10.13 Internal Over Temperature fault - RO
        uint16_t OWF                : 1;    // 0x10.14 Open-wire fault - RO
        uint16_t REGF               : 1;    // 0x10.15 REG Fault V3P3 and V2P5 pins - RO
    } value_b;

} u_raa489220_fault_reg_t;

/* 0x11 Masks and Status Register */
typedef union u_raa489220_mask_stat_reg
{
    uint16_t value;

    struct
    {
        uint16_t CHRGI              : 1;    // 0x11.0  Charge Current Indicator - RO
        uint16_t DCHRGI             : 1;    // 0x11.1  Discharge Current Indicator - RO
        uint16_t LD_PRESI           : 1;    // 0x11.2  Load Present Status - RO
        uint16_t CH_PRESI           : 1;    // 0x11.3  Charge Present Status - RO
        uint16_t IEOC               : 1;    // 0x11.4  Current End of Charge - RO
        uint16_t VEOC               : 1;    // 0x11.5  Voltage End of Charge - RO
        uint16_t VTEMP_STATUS       : 1;    // 0x11.6  VTEMP Status - RO
        uint16_t BUSY               : 1;    // 0x11.7  Busy - RO
        uint16_t V3P3OK             : 1;    // 0x11.8  V3P3 OK - R/W
        uint16_t RSV_9_10           : 2;    // 0x11.9...0x11.10 Reserved
        uint16_t CH_LD_PRESI_MASK   : 1;    // 0x11.11 Charger Present and Load Present Mask - R/W
        uint16_t RSV_12             : 1;    // 0x11.12 Reserved
        uint16_t V_I_EOC_MASK       : 1;    // 0x11.13 VEOC/IEOC Mask - R/W
        uint16_t HVGPIO_MASK        : 1;    // 0x11.14 HVPIO Mask - R/W
        uint16_t BUSY_MASK          : 1;    // 0x11.15 Busy Mask - R/W
    } value_b;

} u_raa489220_mask_stat_reg_t;

/* 0x40 System Configuration 1 Register */
typedef union u_raa489220_config1_reg
{
    uint16_t value;

    struct
    {
        uint16_t LCFET_EN           : 1;    // 0x40.0  Turn on the CFET - R/W
        uint16_t LDFET_EN           : 1;    // 0x40.1  Turn on the DFET - R/W
        uint16_t ALERT_ASSERT       : 1;    // 0x40.2  Enable test the ALERT pin - R/W
        uint16_t HVGPIO_ASSERT      : 1;    // 0x40.3  HVGPIO assert - R/W
        uint16_t RSV1               : 1;    // 0x40.4  Reserved
        uint16_t GO2LPM             : 1;    // 0x40.5  Go to Low Power Mode - R/W
        uint16_t CLR_ALL_FAULTS     : 1;    // 0x40.6  Clear all faults and counters - R/W
        uint16_t SOFT_RESET         : 1;    // 0x40.7  Soft reset - R/W
        uint16_t LCFET_STAT         : 1;    // 0x40.8  LCFET pin status - RO
        uint16_t LDFET_STAT         : 1;    // 0x40.9  LDFET pin status - RO
        uint16_t ALERT_STAT         : 1;    // 0x40.10 Alert pin status - RO
        uint16_t HVGPIO_STAT        : 1;    // 0x40.11 HVGPIO pin status - RO
        uint16_t RSV2               : 4;    // 0x40.12...0x40.15 Reserved
    } value_b;

} u_raa489220_config1_reg_t;

/* 0x41 Measure Select Register */
typedef union u_raa489220_meas_sel_reg
{
    uint16_t value;

    struct
    {
        uint16_t MEAS_SELECT        : 5;    // 0x41.0...0x41.4 Selects which measurements are made - R/W
        uint16_t RSV_5_6            : 2;    // 0x41.5...0x41.6 Reserved
        uint16_t TRIGGER_MEAS       : 1;    // 0x41.7  Trigger measurement - R/W
        uint16_t RSV_8_14           : 7;    // 0x41.8...0x41.14 Reserved
        uint16_t BUSY               : 1;    // 0x41.15 Status of current measurement - RO
    } value_b;

} u_raa489220_meas_sel_reg_t;

/* 0x42 System Configuration 2 Register */
typedef union u_raa489220_config2_reg
{
    uint16_t value;

    struct
    {
        uint16_t VC1_ISRC           : 1;    // 0x42.0  Turns on Open wire current source between VC1 and VSS - R/W
        uint16_t VC2_ISRC           : 1;    // 0x42.1  Turns on Open wire current source between VC2 and VSS - R/W
        uint16_t VC3_ISRC           : 1;    // 0x42.2  Turns on Open wire current source between VC3 and VSS - R/W
        uint16_t VC4_ISRC           : 1;    // 0x42.3  Turns on Open wire current source between VC4 and VSS - R/W
        uint16_t VC5_ISRC           : 1;    // 0x42.4  Turns on Open wire current source between VC5 and VSS - R/W
        uint16_t VC6_ISRC           : 1;    // 0x42.5  Turns on Open wire current source between VC5 and VSS - R/W
        uint16_t VC7_ISRC           : 1;    // 0x42.6  Turns on Open wire current source between VC5 and VSS - R/W
        uint16_t VC8_ISRC           : 1;    // 0x42.7  Turns on Open wire current source between VC5 and VSS - R/W
        uint16_t VC9_ISRC           : 1;    // 0x42.8  Turns on Open wire current source between VC5 and VSS - R/W
        uint16_t VC10_ISRC          : 1;    // 0x42.9  Turns on Open wire current source between VC5 and VSS - R/W
        uint16_t RSV_10_12          : 3;    // 0x42.10...0x40.12 Reserved
        uint16_t LP_REG             : 1;    // 0x42.13 Enable strong regulator in Low Power mode - R/W
        uint16_t SELF_TEST_EN       : 1;    // 0x42.14 Self Test enable - R/W
        uint16_t VTEMP_EN           : 1;    // 0x42.15 VTEMP output enable - R/W
    } value_b;

} u_raa489220_config2_reg_t;

/* 0x80 System Configuration 3 Register */
typedef union u_raa489220_config3_reg
{
    uint16_t value;

    struct
    {
        uint16_t PROD_ID            : 7;    // 0x80.0...0x80.6 Product ID - R/W
        uint16_t SLEEP_EN           : 1;    // 0x80.7  Retain register settings in LP mode - R/W
        uint16_t AUTO_SCAN_EN       : 1;    // 0x80.8  Enable Auto SCAN - R/W
        uint16_t FAULT_DELAY        : 2;    // 0x80.9...0x80.10  Time delay in Auto SCAN if a fault is detected - R/W
        uint16_t SCAN_DELAY         : 2;    // 0x80.11...0x80.12 Time delay between scans in Auto SCAN - R/W
        uint16_t COMM_TIMEOUT       : 2;    // 0x80.13...0x80.14 Communication Time Out - R/W
        uint16_t OW_EN              : 1;    // 0x80.15 Enable Open Wire Test - R/W
    } value_b;

} u_raa489220_config3_reg_t;

/* 0x81 OV and EOC Thresholds Register */
typedef union u_raa489220_ov_eoc_th_reg
{
    uint16_t value;

    struct
    {
        uint16_t OVLO_TH            : 4;    // 0x81.0...0x81.3  Vcell overvoltage lockout threshold - R/W
        uint16_t OV_HYST            : 3;    // 0x81.4...0x81.6  Vcell overvoltage hysteresis - R/W
        uint16_t IEOC               : 2;    // 0x81.7...0x81.8  Current End of Charge threshold - R/W
        uint16_t OV_TH              : 7;    // 0x81.9...0x81.15 Vcell overvoltage threshold - R/W
    } value_b;

} u_raa489220_ov_eoc_th_reg_t;

/* 0x82 DVcellOV and UV Thresholds Register */
typedef union u_raa489220_dvov_uv_th_reg
{
    uint16_t value;

    struct
    {
        uint16_t UVLO_TH            : 4;    // 0x82.0...0x82.3   Vcell undervoltage lockout threshold - R/W
        uint16_t UV_HYST            : 3;    // 0x82.4...0x82.6   Vcell undervoltage hysteresis - R/W
        uint16_t CPWR               : 1;    // 0x82.7 Power FET configuration - R/W
        uint16_t UV_TH              : 4;    // 0x80.8...0x80.11  Vcell undervoltage threshold - R/W
        uint16_t DVOV_TH            : 4;    // 0x80.12...0x80.15 Delta Vcell Over Voltage threshold - R/W
    } value_b;

} u_raa489220_dvov_uv_th_reg_t;

/* 0x83 SCC, OT/UT Thresholds Register */
typedef union u_raa489220_scc_ot_ut_reg
{
    uint16_t value;

    struct
    {
        uint16_t CUT                : 3;    // 0x83.0...0x83.2   Charge unter-temperature limit - R/W
        uint16_t COT                : 3;    // 0x83.3...0x83.5   Charge over-temperature limit - R/W
        uint16_t THERM_EN           : 2;    // 0x83.6...0x83.7   Thermistor enable select - R/W
        uint16_t DUT                : 3;    // 0x83.8...0x83.10  Discharge unter-temperature limit - R/W
        uint16_t DOT                : 3;    // 0x83.11...0x83.13 Discharge over-temperature limit - R/W
        uint16_t SCC_DELAY          : 2;    // 0x83.14...0x83.15 Short Circuit Delay - R/W
    } value_b;

} u_raa489220_scc_ot_ut_reg_t;

/* 0x84 Current Thresholds Register */
typedef union u_raa489220_current_th_reg
{
    uint16_t value;

    struct
    {
        uint16_t COC_TH             : 6;    // 0x83.0...0x83.5   Charge overcurrent threshold - R/W
        uint16_t DOC_TH             : 6;    // 0x83.6...0x83.11  Discharge overcurrent threshold - R/W
        uint16_t SCC_TH             : 3;    // 0x83.12...0x83.14 Short-Circuit Current threshold - R/W
        uint16_t OC_DELAY           : 1;    // 0x83.15 Discharge/Charge overcurrent delay - R/W
    } value_b;

} u_raa489220_current_th_reg_t;

/* 0xFD Die Revision Register */
typedef union u_raa489220_die_revision_reg
{
    uint16_t value;

    struct
    {
        uint16_t REVISION           : 4;    // 0xFD.0...0xFD.3  Die Revision
        uint16_t RSV_4_15           : 12;   // 0xFD.4...0xFD.15 Reserved

    } value_b;

} u_raa489220_die_revision_reg_t;

/* 0xFE Manufacturing ID Register */
typedef union u_raa489220_manufacturing_id_reg
{
    uint16_t value;

    struct
    {
        uint16_t MAN_ID             : 8;    // 0xFE.0...0xFE.7  Manufacturing ID
        uint16_t RSV_8_15           : 8;    // 0xFE.8...0xFE.15 Reserved

    } value_b;

} u_raa489220_manufacturing_id_reg_t;

/* 0xFF Device ID Register */
typedef union u_raa489220_device_id
{
    uint16_t value;

    struct
    {
        uint16_t DEV_ID             : 8;    // 0xFF.0...0xFE.7 Device ID
        uint16_t RSV_8_15           : 8;    // 0xFF.8...0xFE.15 Reserved

    } value_b;

} u_raa489220_device_id_t;


/** RAA489220 fault data structure */
typedef struct st_raa489220_faults
{
    bool    flt_charge_over_current;                ///< Charge Overcurrent Fault
    bool    flt_discharge_over_current;             ///< Discharge Overcurrent Fault
    bool    flt_short_circuit;                      ///< Short-Circuit Fault
    bool    flt_under_temeprature;                  ///< Under-Temperature Fault
    bool    flt_over_temeprature;                   ///< Over-Temperature Fault
    bool    flt_undervoltage;                       ///< Undervoltage Fault
    bool    flt_overvoltage;                        ///< Overvoltage Fault
    bool    flt_delta_cell_ov;                      ///< Delta Cell Voltage Fault
    bool    flt_comm_timeout;                       ///< Communication Time Out
    bool    flt_self_test;                          ///< Self Test Fault
    bool    flt_lock_out;                           ///< Lockout (voltage) Fault
    bool    flt_vtemp;                              ///< Open-wire on temperature pins
    bool    flt_internal_over_temp;                 ///< Internal Over Temperature Fault
    bool    flt_open_wire;                          ///< Open-Wire Fault
    bool    flt_regulator;                          ///< Under/over voltage conditions at the V3P3 and V2P5 pins
} st_raa489220_faults_t;

/** RAA489220  encountered in software faults structure */
typedef struct raa489220_faults_sw
{
    u_raa489220_fault_reg_t faults_sw;              ///< BFE faults from software
    uint8_t ov_cntr;                                ///< Overvoltage fault counter
    uint8_t ovlo_cntr;                              ///< Overvoltage lockout fault counter
    uint8_t uv_cntr;                                ///< Undervoltage fault counter
    uint8_t uvlo_cntr;                              ///< Undervoltage lockout fault counter
    uint8_t docf_cntr;                              ///< Discharge overcurrent fault counter
    uint8_t cocf_cntr;                              ///< Charge overcurrent fault counter
    uint8_t dot_cntr;                               ///< Discharge over-temperature fault counter
    uint8_t cot_cntr;                               ///< Charge over-temperature fault counter
    uint8_t dut_cntr;                               ///< Discharge under-temperature fault counter
    uint8_t cut_cntr;                               ///< Charge under-temperature fault counter
    uint8_t v3p3_cntr;                              ///< V3P3 out of range fault counter
    uint8_t v2p5_cntr;                              ///< V2P5 out of range fault counter
    uint8_t vbg2_cntr;                              ///< VBG2 out of range fault counter
} raa489220_faults_sw_t;

/** RAA489220 measured cells voltages data structure */
typedef union u_raa489220_vcells_meas
{
    float vector[11];

    struct
    {
        float v_cell_1;                             ///< Cell 1 voltage [V]
        float v_cell_2;                             ///< Cell 2 voltage [V]
        float v_cell_3;                             ///< Cell 3 voltage [V]
        float v_cell_4;                             ///< Cell 4 voltage [V]
        float v_cell_5;                             ///< Cell 5 voltage [V]
        float v_cell_6;                             ///< Cell 6 voltage [V]
        float v_cell_7;                             ///< Cell 7 voltage [V]
        float v_cell_8;                             ///< Cell 8 voltage [V]
        float v_cell_9;                             ///< Cell 9 voltage [V]
        float v_cell_10;                            ///< Cell 10 voltage [V]
        float v_pack;                               ///< Pack voltage [V]
    } measurements;
} u_raa489220_vcells_meas_t;

/** RAA489220 measured temperatures data structure */
typedef union u_raa489220_temps_meas
{
    float vector[3];

    struct
    {
        float v_therm_1;                            ///< Thermistor 1 voltage [V]
        float v_therm_2;                            ///< Thermistor 2 voltage [V]
        float v_temp;                               ///< VTEMP pin voltage [V]
    } measurements;
} u_raa489220_temps_meas_t;

/** RAA489220 measure all data structure */
typedef union u_raa489220_all_meas
{
    float vector[19];

    struct
    {
        float v_cell_1;                             ///< Cell 1 voltage [V]
        float v_cell_2;                             ///< Cell 2 voltage [V]
        float v_cell_3;                             ///< Cell 3 voltage [V]
        float v_cell_4;                             ///< Cell 4 voltage [V]
        float v_cell_5;                             ///< Cell 5 voltage [V]
        float v_cell_6;                             ///< Cell 6 voltage [V]
        float v_cell_7;                             ///< Cell 7 voltage [V]
        float v_cell_8;                             ///< Cell 8 voltage [V]
        float v_cell_9;                             ///< Cell 9 voltage [V]
        float v_cell_10;                            ///< Cell 10 voltage [V]
        float v_pack;                               ///< Pack voltage [V]
        float v_pin_v3p3;                           ///< Pin V3P3 voltage [V]
        float v_pin_v2p5;                           ///< Pin V2P5 voltage [V]
        float v_ref_bg2;                            ///< Second Band Gap reference voltage [V]
        float v_temp;                               ///< VTEMP pin voltage [V]
        float v_therm_1;                            ///< Thermistor 1 voltage [V]
        float v_therm_2;                            ///< Thermistor 2 voltage [V]
        float i_discharge;                          ///< Discharge current [A]
        float i_charge;                             ///< Charge current [A]
    } measurements;
} u_raa489220_all_meas_t;

/** RAA489220 single system scan data structure */
typedef union u_raa489220_sys_scan_meas
{
    float vector[7];

    struct
    {
        float v_pack;                               ///< Pack voltage [V]
        float v_cell_max;                           ///< Maximum cell voltage [V]
        float v_cell_min;                           ///< Minimum cell voltage [V]
        float v_therm_1;                            ///< Thermistor 1 voltage [V]
        float v_therm_2;                            ///< Thermistor 2 voltage [V]
        float i_discharge;                          ///< Discharge current [A]
        float i_charge;                             ///< Charge current [A]
    } measurements;
} u_raa489220_sys_scan_meas_t;

/** Battery Front End measure select. */
typedef enum e_raa489220_meas_select
{
    BFE_MEAS_SINLE_SYS_SCAN         = 0x00,         ///< One Complete System Scan with SELF TEST and Thermistor Measurements.
    BFE_MEAS_VTEMP                  = 0x01,         ///< Measure the voltage at the VTEMP pin.
    BFE_MEAS_V3P3                   = 0x02,         ///< Measure the voltage at the V3P3 pin.
    BFE_MEAS_V2P5                   = 0x03,         ///< Measure the voltage at the V2P5 pin.
    BFE_MEAS_VBG2                   = 0x04,         ///< Measure the voltage at the VBG2 pin ????
    BFE_MEAS_INTERNAL_TEMP          = 0x05,         ///< Measures the voltage at the Internal temperature sensor.
    BFE_MEAS_THERM_1                = 0x06,         ///< Measures the voltage at the THERM1 Pin.
    BFE_MEAS_THERM_2                = 0x07,         ///< Measures the voltage at the THERM2 Pin.
    BFE_MEAS_VPACK                  = 0x08,         ///< Measure pack voltage.
    BFE_MEAS_VOLT_CELL_1            = 0x09,         ///< Measure Cell 1 voltage.
    BFE_MEAS_VOLT_CELL_2            = 0x0A,         ///< Measure Cell 2 voltage.
    BFE_MEAS_VOLT_CELL_3            = 0x0B,         ///< Measure Cell 3 voltage.
    BFE_MEAS_VOLT_CELL_4            = 0x0C,         ///< Measure Cell 4 voltage.
    BFE_MEAS_VOLT_CELL_5            = 0x0D,         ///< Measure Cell 5 voltage.
    BFE_MEAS_VOLT_CELL_6            = 0x0E,         ///< Measure Cell 6 voltage.
    BFE_MEAS_VOLT_CELL_7            = 0x0F,         ///< Measure Cell 7 voltage.
    BFE_MEAS_VOLT_CELL_8            = 0x10,         ///< Measure Cell 8 voltage.
    BFE_MEAS_VOLT_CELL_9            = 0x11,         ///< Measure Cell 9 voltage.
    BFE_MEAS_VOLT_CELL_10           = 0x12,         ///< Measure Cell 10 voltage.
    BFE_MEAS_IPACK_DISCH            = 0x16,         ///< Measure the voltage between DCSP and CSN pins.
    BFE_MEAS_IPACK_CH               = 0x17,         ///< Measure the voltage between CCSP and CSN pins.
} e_raa489220_meas_select_t;

/** Battery Front End measurement trigger options for trigger measurement bit in Measure Select Register. */
typedef enum e_raa489220_meas_trigger
{
     BFE_CLEAR_TRIGGER              = 0x00,         ///< Clear measurement trigger.
     BFE_SET_TRIGGER                = 0x01,         ///< Set measurement trigger.
} e_raa489220_meas_trigger_t;

/** Battery Front End VTEMP enable options in System Config 2 Register. */
typedef enum e_raa489220_vtemp_trigger
{
     BFE_DISABLE_VTEMP              = 0x00,         ///< Disable VTEMP pin voltage.
     BFE_ENABLE_VTEMP               = 0x01,         ///< Enable VTEMP pin voltage.
} e_raa489220_vtemp_trigger_t;

/** Battery Front End low power mode settings. */
typedef enum e_raa489220_lp_mode
{
     BFE_SHIP_MODE                  = 0x00,         ///< Lowest power; Weak regulators; Register values are not retained.
     BFE_LP_MODE_STRONG_REG         = 0x11,         ///< Strong regulators; Register values are retained.
} e_raa489220_lp_mode_t;

/** RAA489220 thermistor enable. */
/* Please, refer to RAA489220 Datasheet! */
typedef enum e_raa489220_therm_conf
{
    BFE_THERM_OFF                   = 0x00,         ///< No thermistor measurements
    BFE_THERM_1                     = 0x01,         ///< Measure Thermistor 1 only
    BFE_THERM_1_2                   = 0x02,         ///< Measure Thermistor 1 & 2
} e_raa489220_therm_conf_t;

/** RAA489220 communication timeout. */
/* Please, refer to RAA489220 Datasheet! */
typedef enum e_raa489220_comm_tout
{
    BFE_COMM_TO_OFF                 = 0x00,         ///< Communication timeout off
    BFE_COMM_TO_0_1S                = 0x01,         ///< Communication timeout 100ms
    BFE_COMM_TO_1S                  = 0x02,         ///< Communication timeout 1s
    BFE_COMM_TO_5S                  = 0x03,         ///< Communication timeout 5s
} e_raa489220_comm_tout_t;

/** RAA489220 ALERT pin internal connections configuration. */
/* Please, refer to RAA489220 Datasheet! */
typedef enum e_raa489220_alert_cfg
{
    BFE_ALRT_NORMAL                 = 0x00,         ///< Use normal ALERT pin functionality
    BFE_ALRT_BUSY_CONNECT           = 0x01,         ///< Connect Busy Mask bit to the ALERT pin
    BFE_ALRT_V_I_EOC_CONNECT        = 0x02,         ///< Connect the status of the VEOC and IEOC bits to the ALERT pin
    BFE_ALRT_CH_LD_PRES_CONNECT     = 0x04,         ///< Connect the status of the CH PRESI and LD PRESI bits to the ALERT pin
} e_raa489220_alert_cfg_t;

/** RAA489220 scan delay. */
/* Please, refer to RAA489220 Datasheet! */
typedef enum e_raa489220_scan_delay
{
    BFE_SCAN_DELAY_0S               = 0x00,         ///< Scan Delay 0s
    BFE_SCAN_DELAY_0_1S             = 0x01,         ///< Scan Delay 100ms
    BFE_SCAN_DELAY_0_5S             = 0x02,         ///< Scan Delay 500ms
    BFE_SCAN_DELAY_1S               = 0x03,         ///< Scan Delay 1s
} e_raa489220_scan_delay_t;

/** RAA489220 fault delay. */
/* Please, refer to RAA489220 Datasheet! */
typedef enum e_raa489220_fault_delay
{
    BFE_FAULT_DELAY_OFF             = 0x00,         ///< No delay
    BFE_FAULT_DELAY_0_1S            = 0x01,         ///< Fault Delay 100ms
    BFE_FAULT_DELAY_1S              = 0x02,         ///< Fault Delay 1s
    BFE_FAULT_DELAY_5S              = 0x03,         ///< Fault Delay 5s
} e_raa489220_fault_delay_t;

/** RAA489220 charge/discharge overcurrent delay. */
/* Please, refer to RAA489220 Datasheet! */
typedef enum e_raa489220_oc_delay
{
    BFE_OC_USE_SCAN_DELAY = 0,                      ///< Use Scan Delay for Charge/Discharge Overcurrent detection
    BFE_OC_USE_FAULT_DELAY                          ///< Use Fault Delay for Charge/Discharge Overcurrent detection
} e_raa489220_oc_delay_t;

/** RAA489220 short-circuit delay. */
/* Please, refer to RAA489220 Datasheet! */
typedef enum e_raa489220_sc_delay
{
    BFE_SC_DELAY_0FF                = 0x00,         ///< No short circuit delay
    BFE_SC_DELAY_0_1MS              = 0x01,         ///< Short circuit delay 0.1ms
    BFE_SC_DELAY_1MS                = 0x02,         ///< Short circuit delay 1ms
    BFE_SC_DELAY_10MS               = 0x03,         ///< Short circuit delay 10ms
} e_raa489220_sc_delay_t;

/** RAA489220 short-circuit threshold. */
/* Please, refer to RAA489220 Datasheet! */
typedef enum e_raa489220_sc_th
{
    BFE_SC_THLD_12_5MV              = 0x00,         ///< Short circuit threshold 12.5mV
    BFE_SC_THLD_25MV                = 0x01,         ///< Short circuit threshold 25mV
    BFE_SC_THLD_50MV                = 0x02,         ///< Short circuit threshold 50mV
    BFE_SC_THLD_100MV               = 0x03,         ///< Short circuit threshold 100mV
    BFE_SC_THLD_200MV               = 0x04,         ///< Short circuit threshold 200mV
} e_raa489220_sc_th_t;

/** RAA489220 current end of charge threshold. */
/* Please, refer to RAA489220 Datasheet! */
typedef enum e_raa489220_ieoc_th
{
    BFE_IEOC_THLD_0_65MV            = 0x00,         ///< IEOC threshold 0.65mV
    BFE_IEOC_THLD_0_70MV            = 0x01,         ///< IEOC threshold 0.70mV
    BFE_IEOC_THLD_1MV               = 0x02,         ///< IEOC threshold 1mV
    BFE_IEOC_THLD_3MV               = 0x03,         ///< IEOC threshold 3mV
} e_raa489220_ieoc_th_t;

/** RAA489220 Vcell Overvoltage hysteresis. */
/* Please, refer to RAA489220 Datasheet! */
typedef enum e_raa489220_ov_hyst
{
    BFE_CELL_OV_HYST_25MV           = 0x00,         ///< Vcell OV hysteresis 25mV
    BFE_CELL_OV_HYST_50MV           = 0x01,         ///< Vcell OV hysteresis 50mV
    BFE_CELL_OV_HYST_100MV          = 0x02,         ///< Vcell OV hysteresis 100mV
    BFE_CELL_OV_HYST_200MV          = 0x03,         ///< Vcell OV hysteresis 200mV
    BFE_CELL_OV_HYST_250MV          = 0x04,         ///< Vcell OV hysteresis 250mV
    BFE_CELL_OV_HYST_300MV          = 0x05,         ///< Vcell OV hysteresis 300mV
    BFE_CELL_OV_HYST_350MV          = 0x06,         ///< Vcell OV hysteresis 350mV
    BFE_CELL_OV_HYST_400MV          = 0x07,         ///< Vcell OV hysteresis 400mV
} e_raa489220_ov_hyst_t;

/** RAA489220 Vcell Undervoltage hysteresis. */
/* Please, refer to RAA489220 Datasheet! */
typedef enum e_raa489220_uv_hyst
{
    BFE_CELL_UV_HYST_100MV          = 0x00,         ///< Vcell UV hysteresis 100mV
    BFE_CELL_UV_HYST_200MV          = 0x01,         ///< Vcell UV hysteresis 200mV
    BFE_CELL_UV_HYST_300MV          = 0x02,         ///< Vcell UV hysteresis 300mV
    BFE_CELL_UV_HYST_400MV          = 0x03,         ///< Vcell UV hysteresis 400mV
    BFE_CELL_UV_HYST_500MV          = 0x04,         ///< Vcell UV hysteresis 500mV
    BFE_CELL_UV_HYST_600MV          = 0x05,         ///< Vcell UV hysteresis 600mV
    BFE_CELL_UV_HYST_700MV          = 0x06,         ///< Vcell UV hysteresis 700mV
    BFE_CELL_UV_HYST_800MV          = 0x07,         ///< Vcell UV hysteresis 800mV
} e_raa489220_uv_hyst_t;

/** RAA489220 charge over-temperature threshold. */
/* Please, refer to RAA489220 Datasheet! */
typedef enum e_raa489220_cot_th
{
    BFE_COT_THLD_0_678V             = 0x00,         ///< Charge over-temperature threshold 0.678V
    BFE_COT_THLD_0_590V             = 0x01,         ///< Charge over-temperature threshold 0.590V
    BFE_COT_THLD_0_518V             = 0x02,         ///< Charge over-temperature threshold 0.518V
    BFE_COT_THLD_0_450V             = 0x03,         ///< Charge over-temperature threshold 0.450V
    BFE_COT_THLD_0_390V             = 0x04,         ///< Charge over-temperature threshold 0.390V
    BFE_COT_THLD_0_338V             = 0x05,         ///< Charge over-temperature threshold 0.338V
    BFE_COT_THLD_0_294V             = 0x06,         ///< Charge over-temperature threshold 0.294V
    BFE_COT_THLD_0_258V             = 0x07,         ///< Charge over-temperature threshold 0.258V
} e_raa489220_cot_th_t;

/** RAA489220 charge under-temperature threshold. */
/* Please, refer to RAA489220 Datasheet! */
typedef enum e_raa489220_cut_th
{
    BFE_CUT_THLD_1_622V             = 0x00,         ///< Charge under-temperature threshold 1.622V
    BFE_CUT_THLD_1_526V             = 0x01,         ///< Charge under-temperature threshold 1.526V
    BFE_CUT_THLD_1_422V             = 0x02,         ///< Charge under-temperature threshold 1.422V
    BFE_CUT_THLD_1_314V             = 0x03,         ///< Charge under-temperature threshold 1.314V
    BFE_CUT_THLD_1_198V             = 0x04,         ///< Charge under-temperature threshold 1.198V
    BFE_CUT_THLD_1_086V             = 0x05,         ///< Charge under-temperature threshold 1.086V
    BFE_CUT_THLD_0_974V             = 0x06,         ///< Charge under-temperature threshold 0.974V
    BFE_CUT_THLD_0_866V             = 0x07,         ///< Charge under-temperature threshold 0.866V
} e_raa489220_cut_th_t;

/** RAA489220 discharge over-temperature threshold. */
/* Please, refer to RAA489220 Datasheet! */
typedef enum e_raa489220_dot_th
{
    BFE_DOT_THLD_0_222V             = 0x00,         ///< Discharge over-temperature threshold 0.222V
    BFE_DOT_THLD_0_194V             = 0x01,         ///< Discharge over-temperature threshold 0.194V
    BFE_DOT_THLD_0_170V             = 0x02,         ///< Discharge over-temperature threshold 0.170V
    BFE_DOT_THLD_0_150V             = 0x03,         ///< Discharge over-temperature threshold 0.150V
    BFE_DOT_THLD_0_130V             = 0x04,         ///< Discharge over-temperature threshold 0.130V
    BFE_DOT_THLD_0_114V             = 0x05,         ///< Discharge over-temperature threshold 0.114V
    BFE_DOT_THLD_0_098V             = 0x06,         ///< Discharge over-temperature threshold 0.098V
    BFE_DOT_THLD_0_086V             = 0x07,         ///< Discharge over-temperature threshold 0.086V
} e_raa489220_dot_th_t;

/** RAA489220 discharge under-temperature threshold. */
/* Please, refer to RAA489220 Datasheet! */
typedef enum e_raa489220_dut_th
{
    BFE_DUT_THLD_1_622V             = 0x00,         ///< Discharge under-temperature threshold 1.622V
    BFE_DUT_THLD_1_526V             = 0x01,         ///< Discharge under-temperature threshold 1.526V
    BFE_DUT_THLD_1_422V             = 0x02,         ///< Discharge under-temperature threshold 1.422V
    BFE_DUT_THLD_1_314V             = 0x03,         ///< Discharge under-temperature threshold 1.314V
    BFE_DUT_THLD_1_198V             = 0x04,         ///< Discharge under-temperature threshold 1.198V
    BFE_DUT_THLD_1_086V             = 0x05,         ///< Discharge under-temperature threshold 1.086V
    BFE_DUT_THLD_0_974V             = 0x06,         ///< Discharge under-temperature threshold 0.974V
    BFE_DUT_THLD_0_866V             = 0x07,         ///< Discharge under-temperature threshold 0.866V
} e_raa489220_dut_th_t;

/** RAA489220 HVGPIO Output States. */
/* Please, refer to RAA489220 Datasheet! */
typedef enum e_raa489220_gpio_states
{
    BFE_HVGPIO_LOW = 0,                             ///< High-voltage output is asserted to Vss pin
    BFE_HVGPIO_HIGH_Z,                              ///< High-voltage output is in high-impedance state
} e_raa489220_hvgpio_states_t;

/** RAA489220 HVGPIO control structure. */
typedef struct st_raa489220_gpio_ctrl
{
    e_raa489220_hvgpio_states_t hvgpio_state;       ///< HVGPIO output states
} st_raa489220_hvgpio_ctrl_t;

/** RAA489220 information structure */
typedef struct st_raa489220_info
{
    uint16_t die_revision;                          ///< Die revision
    uint16_t manufacturing_id;                      ///< Manufacturing ID
    uint16_t device_id;                             ///< Device ID
} st_raa489220_info_t;

/** RAA489220 timings structure */
typedef struct raa489220_timings
{
    uint32_t reset_wait_time_ms;                    ///< Reset processing time [ms]
    uint32_t reset_therm2_hold_time_ms;             ///< Hold time for THERM2 pin to trigger a reset [ms]
    uint32_t busy_bit_timeout_us;                   ///< Busy bit read timeout during measurement [us]
} raa489220_timings_t;

/** RAA489220 extended interface configuration */
typedef struct st_raa489220_ext_cfg
{
    /* Shunts */
    const float  r_shunt_charge_mohm;                       ///< Shunt resistance for charge current [mohm]
    const float  r_shunt_discharge_mohm;                    ///< Shunt resistance for discharge current [mohm]

    /* Fault thresholds and delays */
    const float  thold_overvolt_lockout_v;                  ///< Cell overvoltage lockout threshold [V]
                                                            ///< (range: 3.0 - 4.5V, step: 100mV)
    const float  thold_overvolt_v;                          ///< Cell overvoltage threshold [V]
                                                            ///< (range: 3.23 - 4.5V, step: 10mV)
    const float  thold_undervolt_v;                         ///< Cell undervoltage threshold [V]
                                                            ///< (range: 1.5 - 3.0V, step: 100mV)
    const float  thold_undervolt_lockout_v;                 ///< Cell undervoltage lockout threshold [V]
                                                            ///< (range: 1.5 - 3.0V, step: 100mV)
    const float  thold_delta_cell_overvolt_v;               ///< Delta cell overvoltage threshold [V]
                                                            ///< (range: 0.5 - 2.0V, step: 100mV)
    const e_raa489220_ov_hyst_t hyst_overovltage;           ///< Cell overvoltage hysteresis
    const e_raa489220_uv_hyst_t hyst_undervltage;           ///< Cell overvoltage hysteresis

    const e_raa489220_cot_th_t thold_charge_over_temp;      ///< Over-temperature threshold when charging
    const e_raa489220_cut_th_t thold_charge_under_temp;     ///< Under-temperature threshold when charging
    const e_raa489220_dot_th_t thold_discharge_over_temp;   ///< Over-temperature threshold when discharging
    const e_raa489220_dut_th_t thold_discharge_under_temp;  ///< Under-temperature threshold when discharging

    const float  thold_charge_overcurrent_a;                ///< Charge overcurrent threshold [A]
    const float  thold_discharge_overcurrent_a;             ///< Discharge overcurrent threshold [A]
    const e_raa489220_oc_delay_t delay_ch_d_oc;             ///< Discharge/Charge overcurrent delay
    const e_raa489220_sc_th_t thold_sc;                     ///< Short-circuit threshold
    const e_raa489220_sc_delay_t delay_sc;                  ///< Short-circuit delay
    const e_raa489220_ieoc_th_t ieoc_threshold;             ///< End of charge current threshold
    const bool discharge_overcurrent_disable;               ///< Disable discharge overcurrent protection

    const e_raa489220_comm_tout_t  comm_timeout;            ///< Communication timeout

    const bool self_test_enable;                            ///< Include self-testing in single scan sequence.
    const bool open_wire_test_enable;                       ///< Include open wire test in the self-test loop.
    const bool fuse_blow_enable;                            ///< Enable fuse blow on the BFE.

    const e_raa489220_alert_cfg_t alert_pin_cfg;            ///< Alert pin control sources selection

    const e_raa489220_scan_delay_t  scan_delay;             ///< Auto scan interval
    const e_raa489220_fault_delay_t fault_delay;            ///< Auto scan interval when fault condition is detected
    const e_raa489220_lp_mode_t  lp_mode_settings;          ///< Low Power Mode Settings when transition from Auto scan

    /* Pins configuration */
    const bsp_io_port_pin_t pin_alert;                      ///< ALERT pin
    const bsp_io_port_pin_t pin_reset;                      ///< THERM2 pin
} st_raa489220_ext_cfg_t;

/** RAA489220 extended control structure */
typedef struct st_raa489220_ext_ctrl
{
    uint8_t             i2c_slave_address;                  ///< I2C communication slave address of the BFE
    bool                weak_regulator;                     ///< Weak regulator setting state
    raa489220_timings_t timings;                            ///< Structure holding the BFE timings
} st_raa489220_ext_ctrl_t;

/**********************************************************************************************************************
 External global variables
 *********************************************************************************************************************/
extern const bfe_api_t                g_bfe_on_bfe;
extern st_raa489220_registers_t g_raa489220_registers;

/**********************************************************************************************************************
 Exported global functions
 *********************************************************************************************************************/

/***********************************************************************************************************************
 * Public APIs
 **********************************************************************************************************************/

/* Function Name: R_RAA489220_Init */
/******************************************************************************************************************//**
 * @brief This function initializes a BFE.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_Init (st_bfe_ctrl_t * const p_bfe_ctrl, st_bfe_cfg_t const * const p_bfe_cfg);

/* Function Name: R_RAA489220_Deinit */
/******************************************************************************************************************//**
 * @brief This function deinitializes a BFE.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_Deinit (st_bfe_ctrl_t * const p_bfe_ctrl);

/* Function Name: R_RAA489220_Setup */
/******************************************************************************************************************//**
 * @brief This function configures a BFE.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_Setup (st_bfe_ctrl_t * const p_bfe_ctrl, st_bfe_cfg_t const * const p_bfe_cfg);

/* Function Name: R_RAA489220_Reset */
/******************************************************************************************************************//**
 * @brief This function resets and reinitializes a BFE.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_Reset (st_bfe_ctrl_t * const p_bfe_ctrl, e_bfe_reset_type_t type);

/* Function Name: R_RAA489220_ModeSet */
/******************************************************************************************************************//**
 * @brief This function .....
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_ModeSet (st_bfe_ctrl_t * const p_bfe_ctrl, e_bfe_mode_t mode);

/* Function Name: R_RAA489220_ModeRead */
/******************************************************************************************************************//**
 * @brief This function gets the current mode of a BFE.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_ModeRead (st_bfe_ctrl_t * const p_bfe_ctrl, e_bfe_mode_t * const p_mode);

/* Function Name: R_RAA489220_CommTest */
/******************************************************************************************************************//**
 * @brief This function tests the communication between a MCU and a BFE.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_CommTest (st_bfe_ctrl_t * const p_bfe_ctrl);

/* Function Name: R_RAA489220_SelfDiag */
/******************************************************************************************************************//**
 * @brief This function runs a self-test procedure.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_SelfDiag (st_bfe_ctrl_t * const p_bfe_ctrl, e_bfe_diag_option_t option);

/* Function Name: R_RAA489220_MemCheck */
/******************************************************************************************************************//**
 * @brief This function checks the BFE memory for corrupted configuration registers.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_MemCheck (st_bfe_ctrl_t * const p_bfe_ctrl, e_bfe_mem_check_option_t option);

/* Function Name: R_RAA489220_VPackGet */
/******************************************************************************************************************//**
 * @brief This function acquires the total battery pack voltage.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_VPackGet (st_bfe_ctrl_t * const p_bfe_ctrl,
                                  float         * const p_value,
                                  bool                  trigger);

/* Function Name: R_RAA489220_IPackGet */
/******************************************************************************************************************//**
 * @brief This function acquires the battery pack current.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_IPackGet (st_bfe_ctrl_t        * const p_ctrl,
                                  st_bfe_i_pack_meas_t * const p_values,
                                  bool                         trigger);

/* Function Name: R_RAA489220_VCellsGet */
/******************************************************************************************************************//**
 * @brief This function acquires the minimum, maximum and delta cell voltage.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_VCellsGet (st_bfe_ctrl_t    * const p_bfe_ctrl,
                                   bfe_vcell_meas_t * const p_values,
                                   bool                     trigger);

/* Function Name: R_RAA489220_Temps */
/******************************************************************************************************************//**
 * @brief This function acquires thermistors' voltages.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_Temps (st_bfe_ctrl_t   * const p_bfe_ctrl,
                               bfe_temp_meas_t * const p_values,
                               bool                    trigger);

/* Function Name: R_RAA489220_AllGet */
/******************************************************************************************************************//**
 * @brief This function acquires all voltages, currents and temperatures.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_AllGet (st_bfe_ctrl_t  * const p_bfe_ctrl,
                                bfe_all_meas_t * const p_values,
                                bool                   trigger);

/* Function Name: R_RAA489220_VMixGet */
/******************************************************************************************************************//**
 * @brief This function acquires .......
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_SysScanGet (st_bfe_ctrl_t    * const p_bfe_ctrl,
                                    bfe_other_meas_t * const p_values,
                                    bool                     trigger);

/* Function Name: R_RAA489220_FaultsAllRead */
/******************************************************************************************************************//**
 * @brief This function reads the fault register.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_FaultsAllRead (st_bfe_ctrl_t * const p_bfe_ctrl, bfe_faults_t * const p_faults);

/* Function Name: R_RAA489220_FaultsCheck */
/******************************************************************************************************************//**
 * @brief This function checks the alert pin.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_FaultsCheck (st_bfe_ctrl_t * const p_bfe_ctrl);

/* Function Name: R_RAA489220_FaultsAllClear */
/******************************************************************************************************************//**
 * @brief This function clears all faults.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_FaultsAllClear (st_bfe_ctrl_t * const p_bfe_ctrl, bool * const p_success);

/* Function Name: R_RAA489220_CellBalanceCtrl */
/******************************************************************************************************************//**
 * @brief This function selects cells and enables or inhibits cell balancing.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_CellBalanceCtrl (st_bfe_ctrl_t       * const p_bfe_ctrl,
                                         bfe_cb_cfg_t        * const p_bal_cfg,
                                         e_bfe_process_ctrl_t        ctrl_option);

/* Function Name: R_RAA489220_IsCellBalancing */
/******************************************************************************************************************//**
 * @brief This function checks if cell balancing is active.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_IsCellBalancing (st_bfe_ctrl_t * const p_bfe_ctrl);

/* Function Name: R_RAA489220_AutoScanCtrl */
/******************************************************************************************************************//**
 * @brief This function controls the Continuous Scan function of the BFE.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_ContScanCtrl (st_bfe_ctrl_t       * const p_bfe_ctrl,
                                      bfe_scan_cont_cfg_t * const p_scan_cfg,
                                      e_bfe_process_ctrl_t        ctrl_option);

/* Function Name: R_RAA489220_WatchdogCtrl */
/******************************************************************************************************************//**
 * @brief This function enables or disables the watchdog timer and selects time.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_WatchdogCtrl (st_bfe_ctrl_t * const p_bfe_ctrl, bfe_watchdog_ctrl_t * const p_options);

/* Function Name: R_RAA489220_FETsCtrl */
/******************************************************************************************************************//**
 * @brief This function controls power FETs.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_FETsCtrl (st_bfe_ctrl_t   * const p_bfe_ctrl,
                                  uint8_t                 group_num,
                                  e_bfe_cfet_state_t      c_fet_state,
                                  e_bfe_dfet_state_t      d_fet_state);

/* Function Name: R_RAA489220_HVGPIOCtrl */
/******************************************************************************************************************//**
 * @brief This function controls HVGPIO pin output level.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_HVGPIOCtrl (st_bfe_ctrl_t * const p_bfe_ctrl, bfe_gpio_ctrl_t * const p_control_options);

/* Function Name: R_RAA489220_RegisterRead */
/******************************************************************************************************************//**
 * @brief This function reads a register of a BFE.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_RegisterRead (st_bfe_ctrl_t * const p_ctrl, bfe_register_t * const p_register);

/* Function Name: R_RAA489220_RegisterWrite */
/******************************************************************************************************************//**
 * @brief This function writes to a register of a BFE.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_RegisterWrite (st_bfe_ctrl_t * const p_ctrl, bfe_register_t * const p_register);

#endif /* BFE_R_RAA489220_H_ */

/*******************************************************************************************************************//**
 * @} (end defgroup RAA489220)
 **********************************************************************************************************************/
