/**********************************************************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
 * other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
 * applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
 * EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
 * SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO
 * THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
 * this software. By using this software, you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 *
 * Copyright (C) 2021 Renesas Electronics Corporation. All rights reserved.
 *********************************************************************************************************************/
/******************************************************************************************************************//**
 * @file          bms.c
 * @version       1.00
 * @brief         Contains functions implementing battery management algorithms.
 *********************************************************************************************************************/
/**********************************************************************************************************************
 * History : DD.MM.YYYY Version  Description
 *         : 07.07.2022 1.00     First Release.
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Includes   <System Includes> , "Project Includes"
 *********************************************************************************************************************/
/* Generic headers */
#include <stdint.h>
#include <stdbool.h>
#include <string.h>

/* BFE related headers */
#include <r_bms.h>
#include <r_bms_cfg.h>

/**********************************************************************************************************************
 Macro definitions
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Local Typedef definitions
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Exported global variables
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Private (static) variables and functions
 *********************************************************************************************************************/
static volatile bool s_periodic_timer_event_flag = false;

e_bms_err_t g_bms_err = BMS_SUCCESS; // Error code
e_bfe_err_t g_bfe_err = BFE_SUCCESS; // Error code

/* USB related variables */
usb_status_t g_usb_event = USB_STATUS_NONE;
usb_setup_t  g_usb_setup = {0};

static usb_pcdc_linecoding_t s_line_coding =
{
 .dw_dte_rate   = 115200,
 .b_char_format = STOP_BITS_1,
 .b_parity_type = PARITY_NONE,
 .b_data_bits   = DATA_BITS_8,
};

static uint8_t s_usb_rx_buff[USB_READ_BUF_SIZE] = {0};
static char    s_usb_print_dat_buff[USB_LONG_PACKET_SIZE] = {0};

bool g_usb_attach = false;

/* States control variables */
static st_bms_trans_flags_t s_bms_trans_flags  =
{
 .goto_idle           = false,
 .goto_measure        = false,
 .goto_sleep          = false,
 .goto_autoscan       = false,
 .goto_fault          = false,
};

static bool s_carriage_return = false;
static bool s_pow_fets_toggle = false;

/* Banners */
char g_intro[USB_SHORT_PACKET_SIZE] = {INTRO};
char g_main_menu[USB_LONG_PACKET_SIZE] = {MAIN_MENU};
char g_main_menu2[USB_LONG_PACKET_SIZE] = {MAIN_MENU_2};
char g_fets_state[USB_SHORT_PACKET_SIZE] = {FETS_CTRL_1};
char g_fets_state2[USB_SHORT_PACKET_SIZE] = {FETS_CTRL_2};
char g_meas_state[USB_SHORT_PACKET_SIZE] = {MEAS_MSG_1};
char g_autoscan_mode[USB_SHORT_PACKET_SIZE] = {AUTOSCAN_MODE_1};
char g_autoscan_mode2[USB_SHORT_PACKET_SIZE] = {AUTOSCAN_MODE_2};
char g_autoscan_mode3[USB_SHORT_PACKET_SIZE] = {AUTOSCAN_MODE_3};
char g_sleep_mode[USB_SHORT_PACKET_SIZE] = {SLEEP_MODE_1};
char g_sleep_mode2[USB_SHORT_PACKET_SIZE] = {SLEEP_MODE_2};
char g_sleep_mode3[USB_SHORT_PACKET_SIZE] = {SLEEP_MODE_3};
char g_fault_mode[USB_SHORT_PACKET_SIZE] = {FAULTS_MODE};
char g_kit_info[USB_LONG_PACKET_SIZE] = {INFO_BANNER_1};
char g_fault_msg1[USB_SHORT_PACKET_SIZE] = {FAULTS_MSG_1};
char g_fault_msg2[USB_SHORT_PACKET_SIZE] = {FAULTS_MSG_2};
char g_fault_msg3[USB_SHORT_PACKET_SIZE] = {FAULTS_MSG_3};

/* Private functions */
static void        bms_state_machine (void);
static fsp_err_t   usb_event_process (void);
static void        command_handler (uint8_t * p_rx_data);
static fsp_err_t   measured_data_process (u_raa489220_all_meas_t * p_data);
static fsp_err_t   measured_data_process2 (u_raa489220_sys_scan_meas_t * p_data, int index);
static fsp_err_t   print_to_console (char * p_data);
static fsp_err_t   check_for_write_complete (void);
static e_bms_err_t bfe_fault_management (e_bfe_err_t bfe_err, bool clear_fault);
static fsp_err_t   faults_data_process (st_raa489220_faults_t * p_faults);

/**********************************************************************************************************************
 * Function Name: r_bms_main
 * Description  : This is the main battery management system demo function.
 * Arguments    : none
 * Return Value : none
 *********************************************************************************************************************/
void r_bms_main(void)
{
    fsp_err_t fsp_err = FSP_SUCCESS; // Error status

    static timer_info_t s_periodic_info =
    {
     .clock_frequency = RESET_VALUE,
     .count_direction = (timer_direction_t) RESET_VALUE,
     .period_counts = RESET_VALUE,
    };

    /* Open USB instance */
    fsp_err = g_basic0.p_api->open(&g_basic0_ctrl, &g_basic0_cfg);
    if (FSP_SUCCESS != fsp_err)
    {
        TURN_D7_ON // Turn ON RED LED to indicate fatal error
        APP_ERR_TRAP(fsp_err);
    }

    /* Open periodic timer. */
    fsp_err = g_timer_periodic.p_api->open(&g_timer_periodic_ctrl, &g_timer_periodic_cfg);
    if (FSP_SUCCESS != fsp_err)
    {
        TURN_D7_ON // Turn ON RED LED to indicate fatal error
        APP_ERR_TRAP(fsp_err);
    }

    /* Get the timer clock frequency. */
    fsp_err = g_timer_periodic.p_api->infoGet(&g_timer_periodic_ctrl, &s_periodic_info);
    if (FSP_SUCCESS != fsp_err)
    {
        TURN_D7_ON // Turn ON RED LED to indicate fatal error
        APP_ERR_TRAP(fsp_err);
    }

    /* Calculate and set period. */
    uint32_t raw_counts_periodic = (uint32_t) ((TIME_PERIOD_MS_PERIODIC * s_periodic_info.clock_frequency) / 1000);

    fsp_err = g_timer_periodic.p_api->periodSet(&g_timer_periodic_ctrl, raw_counts_periodic);
    if (FSP_SUCCESS != fsp_err)
    {
        TURN_D7_ON // Turn ON RED LED to indicate fatal error
        APP_ERR_TRAP(fsp_err);
    }

    /* Run periodic timer. */
    fsp_err = g_timer_periodic.p_api->start(&g_timer_periodic_ctrl);
    if (FSP_SUCCESS != fsp_err)
    {
        TURN_D7_ON // Turn ON RED LED to indicate fatal error
        APP_ERR_TRAP(fsp_err);
    }

    while(1)
    {
        /* Process the USB event. */
        fsp_err = usb_event_process();

        if (FSP_SUCCESS != fsp_err)
        {
            TURN_D7_ON // Turn ON RED LED to indicate fatal error
            APP_ERR_TRAP(fsp_err);
        }

        /* Process the BMS state machine. */
        if(s_periodic_timer_event_flag == true)
        {
                bms_state_machine();

                s_periodic_timer_event_flag = false;
        }
        else
        {
            ;
        }

     }
}
/**********************************************************************************************************************
 * End of function r_bms_main
 *********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: bms_state_machine
 * Description  : This function is processing the states and transitions between them.
 * Arguments    : none
 * Return Value : none
 *********************************************************************************************************************/
void bms_state_machine(void)
{
    fsp_err_t fsp_err = FSP_SUCCESS; // Error status

    static e_bms_machine_states_t bms_state = BMS_INIT_STATE;

    static uint32_t s_counter_fault_check = 0;
    static uint32_t s_counter_self_diag = 0;
    static uint32_t s_counter_auto_scan = AUTO_SCAN_RESET;
    static int s_index = 0;

    static u_raa489220_all_meas_t      s_meas_data_all = {0};
    static u_raa489220_sys_scan_meas_t s_meas_sys_scan = {0};

    static bool s_do_once = true;
    static bool s_power_fets_on = false;

    /* State machine transition rules. */
    if(s_bms_trans_flags.goto_fault == true)
    {
        s_bms_trans_flags.goto_fault = false; // Clear transition flag

        s_do_once = true;

        bms_state = BMS_FAULT_MODE;
    }
    else if(s_bms_trans_flags.goto_idle == true)
    {
        s_bms_trans_flags.goto_idle = false; // Clear transition flag

        s_do_once = true;

        bms_state = BMS_IDLE_MODE;
    }
    else if((bms_state == BMS_IDLE_MODE) && (s_bms_trans_flags.goto_measure == true))
    {
        s_bms_trans_flags.goto_measure = false; // Clear transition flag

        s_do_once = true;

        bms_state = BMS_SCAN_STATE;
    }
    else if((bms_state == BMS_IDLE_MODE) && (s_bms_trans_flags.goto_sleep == true))
    {
        s_bms_trans_flags.goto_sleep = false; // Clear transition flag

        s_do_once = true;

        bms_state = BMS_SLEEP_MODE;
    }
    else if((bms_state == BMS_IDLE_MODE) && (s_bms_trans_flags.goto_autoscan == true))
    {
        s_bms_trans_flags.goto_autoscan = false; // Clear transition flag

        s_do_once = true;

        bms_state = BMS_AUTOSCAN_MODE;
    }
    else
    {
        ;
    }

    /* Process states. */
    switch(bms_state)
    {
        case BMS_INIT_STATE: // Initialize BMS on start-up
        {
            if(s_do_once == true)
            {
                /* Initialize the Battery Front End. */
                g_bfe_err = g_bfe0.p_api->p_initialize(&g_bfe0_ctrl, &g_bfe0_cfg);
                if(g_bfe_err != BFE_SUCCESS) // Check for error return
                {
                    g_bms_err = BMS_ERR_BFE;
                    s_bms_trans_flags.goto_fault = true; // Go to Fault state to process error.
                    break;
                }

                /* Check if registry defaults are correctly loaded. */
                g_bfe_err = g_bfe0.p_api->p_memoryCheck(&g_bfe0_ctrl, BFE_CHECK_DEF_VALS);
                if(g_bfe_err != BFE_SUCCESS) // Check for error return
                {
                    g_bms_err = BMS_ERR_BFE;
                    s_bms_trans_flags.goto_fault = true; // Go to Fault state to process error.
                    break;
                }

                /* Configure the Battery Front End. */
                g_bfe_err = g_bfe0.p_api->p_setup(&g_bfe0_ctrl, &g_bfe0_cfg);
                if(g_bfe_err != BFE_SUCCESS) // Check for error return
                {
                    g_bms_err = BMS_ERR_BFE;
                    s_bms_trans_flags.goto_fault = true; // Go to Fault state to process error.
                    break;
                }

                /* Test communication. */
                g_bfe_err = g_bfe0.p_api->p_commTest(&g_bfe0_ctrl);
                if(g_bfe_err != BFE_SUCCESS) // Check for error return
                {
                    g_bms_err = BMS_ERR_BFE;
                    s_bms_trans_flags.goto_fault = true; // Go to Fault state to process error.
                    break;
                }

                /* Run self diagnostic. */
                g_bfe_err = g_bfe0.p_api->p_selfDiagnostic(&g_bfe0_ctrl, BFE_FULL_TEST);
                if((g_bfe_err != BFE_SUCCESS) || (g_bfe0_ctrl.is_fault_detected == true)) // Check for error return
                {
                    g_bms_err = BMS_ERR_BFE;
                    s_bms_trans_flags.goto_fault = true; // Go to Fault state to process error.
                    break;
                }

                s_do_once = false;
            }
            else
            {
                ;
            }

            if(s_carriage_return == true)
            {
                s_carriage_return = false;

                /* Print banner info to console. */
                fsp_err = print_to_console(g_intro);
                if (FSP_SUCCESS != fsp_err)
                {
                    TURN_D7_ON // Turn ON RED LED to indicate fatal error
                    APP_ERR_TRAP(fsp_err);
                }

                s_bms_trans_flags.goto_idle = true; // Set transition flag
            }

            break;
        }

        case BMS_IDLE_MODE: // Wait for an user input and run diagnostic periodically
        {
            if((s_do_once == true) || (s_carriage_return == true)) // Send to terminal once on first entry
            {
                /* Print banner info to console. */
                if(s_power_fets_on == true)
                {
                    fsp_err = print_to_console(g_main_menu2);
                    if (FSP_SUCCESS != fsp_err)
                    {
                        TURN_D7_ON // Turn ON RED LED to indicate fatal error
                        APP_ERR_TRAP(fsp_err);
                    }
                }
                else
                {
                    fsp_err = print_to_console(g_main_menu);
                    if (FSP_SUCCESS != fsp_err)
                    {
                        TURN_D7_ON // Turn ON RED LED to indicate fatal error
                        APP_ERR_TRAP(fsp_err);
                    }
                }

                s_do_once = false;
                s_carriage_return = false;
            }
            else
            {
                ;
            }

            /* Toggle Power FETs */
            if(s_pow_fets_toggle == true)
            {
                s_pow_fets_toggle = false;

                s_do_once = true;

                if(s_power_fets_on == true)
                {
                    g_bfe_err = g_bfe0.p_api->p_fetControl(&g_bfe0_ctrl, 1, BFE_CFET_OFF, BFE_DFET_OFF);
                    if(g_bfe_err != BFE_SUCCESS) // Check for error return
                    {
                        g_bms_err = BMS_ERR_BFE;
                        s_bms_trans_flags.goto_fault = true; // Go to Fault state to process error.
                        break;
                    }

                    s_power_fets_on = false;

                    fsp_err = print_to_console(g_fets_state);
                    if (FSP_SUCCESS != fsp_err)
                    {
                        TURN_D7_ON // Turn ON RED LED to indicate fatal error
                        APP_ERR_TRAP(fsp_err);
                    }
                }
                else
                {
                    g_bfe_err = g_bfe0.p_api->p_fetControl(&g_bfe0_ctrl, 1, BFE_CFET_ON, BFE_DFET_ON);
                    if(g_bfe_err != BFE_SUCCESS) // Check for error return
                    {
                        g_bms_err = BMS_ERR_BFE;
                        s_bms_trans_flags.goto_fault = true; // Go to Fault state to process error.
                        break;
                    }

                    s_power_fets_on = true;

                    fsp_err = print_to_console(g_fets_state2);
                    if (FSP_SUCCESS != fsp_err)
                    {
                        TURN_D7_ON // Turn ON RED LED to indicate fatal error
                        APP_ERR_TRAP(fsp_err);
                    }
                }
            }
            else
            {
                ;
            }

            s_counter_self_diag++; // Increment self-diagnostic counter

            /* Self-diagnostic timeout. */
            if(s_counter_self_diag > BMS_SELF_DIAG)
            {
                s_counter_self_diag = 0; // Reset counter

                /* Run self diagnostic. */
                g_bfe_err = g_bfe0.p_api->p_selfDiagnostic(&g_bfe0_ctrl, BFE_FULL_TEST);
                if((g_bfe_err != BFE_SUCCESS) || (g_bfe0_ctrl.is_fault_detected == true)) // Check for error return
                {
                    g_bms_err = BMS_ERR_BFE;
                    s_bms_trans_flags.goto_fault = true; // Go to Fault state to process error.
                    break;
                }
            }
            else
            {
                ;
            }

            break;
        }

        case BMS_SCAN_STATE:
        {
            fsp_err = print_to_console(g_meas_state);
            if (FSP_SUCCESS != fsp_err)
            {
                TURN_D7_ON // Turn ON RED LED to indicate fatal error
                APP_ERR_TRAP(fsp_err);
            }

            /* Clean data structures. */
            memset(&s_meas_data_all, 0, sizeof(s_meas_data_all));

            /* Acquire all voltages and temperatures. */
            g_bfe_err = g_bfe0.p_api->p_allGet(&g_bfe0_ctrl, &s_meas_data_all, true);
            if(g_bfe_err != BFE_SUCCESS) // Check for error return
            {
                g_bms_err = BMS_ERR_BFE;
                s_bms_trans_flags.goto_fault = true; // Go to Fault state to process error.
                break;
            }

            fsp_err = measured_data_process(&s_meas_data_all);
            if (FSP_SUCCESS != fsp_err)
            {
                TURN_D7_ON // Turn ON RED LED to indicate fatal error
                APP_ERR_TRAP(fsp_err);
            }

            s_bms_trans_flags.goto_idle = true; // Set transition flag.

            break;
        }

        case BMS_AUTOSCAN_MODE: // Start auto scan mode
        {
            if(s_do_once == true) // Send to terminal once on first entry
            {
                /* Print banner info to console. */
                fsp_err = print_to_console(g_autoscan_mode);
                if (FSP_SUCCESS != fsp_err)
                {
                    TURN_D7_ON // Turn ON RED LED to indicate fatal error
                    APP_ERR_TRAP(fsp_err);
                }

                s_do_once = false;
            }
            else
            {
                ;
            }

            if(g_bfe0_ctrl.is_cont_scanning == false) // Check if already in auto scan mode
            {
                /* Enable Auto Scan Mode in the BFE. */
                g_bfe_err = g_bfe0.p_api->p_modeSet(&g_bfe0_ctrl, BFE_MODE_AUTOSCAN);
                if(g_bfe_err != BFE_SUCCESS) // Check for error return
                {
                    g_bms_err = BMS_ERR_BFE;
                    s_bms_trans_flags.goto_fault = true; // Go to Fault state to process error.
                    break;
                }

                s_index = 0;

                /* Print banner info to console. */
                fsp_err = print_to_console(g_autoscan_mode2);
                if (FSP_SUCCESS != fsp_err)
                {
                    TURN_D7_ON // Turn ON RED LED to indicate fatal error
                    APP_ERR_TRAP(fsp_err);
                }
            }
            else if(s_carriage_return == true)
            {
                s_carriage_return = false;

                /* Return to Idle Mode. */
                g_bfe_err = g_bfe0.p_api->p_modeSet(&g_bfe0_ctrl, BFE_MODE_IDLE);
                if(g_bfe_err != BFE_SUCCESS) // Check for error return
                {
                    g_bms_err = BMS_ERR_BFE;
                    s_bms_trans_flags.goto_fault = true; // Go to Fault state to process error.
                    break;
                }

                fsp_err = print_to_console(g_autoscan_mode3);
                if (FSP_SUCCESS != fsp_err)
                {
                    TURN_D7_ON // Turn ON RED LED to indicate fatal error
                    APP_ERR_TRAP(fsp_err);
                }

                s_bms_trans_flags.goto_idle = true; // Set transition flag.
            }
            else
            {
                s_counter_fault_check++; // Increment faults-check counter

                /* Fault check timeout. */
                if(s_counter_fault_check > BMS_FAULT_CHECK)
                {
                    s_counter_fault_check = 0; // Reset counter

                    /* Check for faults. */
                    g_bfe_err = g_bfe0.p_api->p_faultsCheck(&g_bfe0_ctrl);
                    if((g_bfe_err != BFE_SUCCESS) || (g_bfe0_ctrl.is_fault_detected == true)) // Check for error return
                    {
                        g_bms_err = BMS_ERR_BFE;
                        s_bms_trans_flags.goto_fault = true; // Go to Fault state to process error.
                        break;
                    }
                }
                else
                {
                    ;
                }

                s_counter_auto_scan++; // Increment auto-scan counter

                /* Self-diagnostic timeout. */
                if(s_counter_auto_scan > BMS_AUTO_SCAN)
                {
                    s_counter_auto_scan = AUTO_SCAN_RESET; // Reset counter

                    /* Read values. */
                    g_bfe_err = g_bfe0.p_api->p_otherGet(&g_bfe0_ctrl, &s_meas_sys_scan, false);
                    if(g_bfe_err != BFE_SUCCESS) // Check for error return
                    {
                        g_bms_err = BMS_ERR_BFE;
                        s_bms_trans_flags.goto_fault = true; // Go to Fault state to process error.
                        break;
                    }

                    s_index++;

                    fsp_err = measured_data_process2(&s_meas_sys_scan, s_index);
                    if (FSP_SUCCESS != fsp_err)
                    {
                        TURN_D7_ON // Turn ON RED LED to indicate fatal error
                        APP_ERR_TRAP(fsp_err);
                    }
                }
                else
                {
                    ;
                }
            }

            break;
        }

        case BMS_SLEEP_MODE:  // Enter sleep mode
        {
            if(s_do_once == true) // Send to terminal once on first entry
            {
                /* Print banner info to console. */
                fsp_err = print_to_console(g_sleep_mode);
                if (FSP_SUCCESS != fsp_err)
                {
                    TURN_D7_ON // Turn ON RED LED to indicate fatal error
                    APP_ERR_TRAP(fsp_err);
                }

                s_do_once = false;
            }
            else
            {
                ;
            }

            if(g_bfe0_ctrl.is_low_power == false) // Check if already in sleep mode
            {
                /* Send sleep command to the BFE. */
                g_bfe_err = g_bfe0.p_api->p_modeSet(&g_bfe0_ctrl, BFE_MODE_LOW_POWER);
                if(g_bfe_err != BFE_SUCCESS) // Check for error return
                {
                    g_bms_err = BMS_ERR_BFE;
                    s_bms_trans_flags.goto_fault = true; // Go to Fault state to process error.
                    break;
                }

                /* Print banner info to console. */
                fsp_err = print_to_console(g_sleep_mode2);
                if (FSP_SUCCESS != fsp_err)
                {
                    TURN_D7_ON // Turn ON RED LED to indicate fatal error
                    APP_ERR_TRAP(fsp_err);
                }
            }
            else if(s_carriage_return == true)
            {
                s_carriage_return = false;

                /* Send a wake up command to the BFE. */
                g_bfe_err = g_bfe0.p_api->p_modeSet(&g_bfe0_ctrl, BFE_MODE_IDLE);
                if(g_bfe_err != BFE_SUCCESS) // Check for error return
                {
                    g_bms_err = BMS_ERR_BFE;
                    s_bms_trans_flags.goto_fault = true; // Go to Fault state to process error.
                    break;
                }

                /* Print banner info to console. */
                fsp_err = print_to_console(g_sleep_mode3);
                if (FSP_SUCCESS != fsp_err)
                {
                    TURN_D7_ON // Turn ON RED LED to indicate fatal error
                    APP_ERR_TRAP(fsp_err);
                }

                s_bms_trans_flags.goto_idle = true; // Set transition flag.
            }
            else
            {
                ;
            }

            break;
        }

        case BMS_FAULT_MODE:  // Process BMS fault
        {
            TURN_D7_ON // Turn ON RED LED to indicate fatal error.

            if(s_do_once == true) // Send to terminal once on first entry
            {
                s_do_once = false;

                /* Print banner info to console. */
                fsp_err = print_to_console(g_fault_mode);
                if (FSP_SUCCESS != fsp_err)
                {
                    APP_ERR_TRAP(fsp_err);
                }

                g_bfe0.p_api->p_fetControl(&g_bfe0_ctrl, 1, BFE_CFET_OFF, BFE_DFET_OFF);

                s_power_fets_on = false;
            }
            else
            {
                ;
            }

            switch(g_bms_err)
            {
                case BMS_SUCCESS:
                {
                    TURN_D7_OFF // Turn off the red LED

                    s_do_once = true;

                    s_bms_trans_flags.goto_idle = true; // Set transition flag.

                    break;
                }

                case BMS_ERR_BFE:
                {
                    /* Call BFE fault management function. */
                    g_bms_err = bfe_fault_management(g_bfe_err, s_carriage_return);

                    s_carriage_return = false;

                    break;
                }

                case BMS_ERR_FSP:
                {
                    /* Print banner info to console. */
                    print_to_console(g_fault_msg1);

                    /* Halt */
                    APP_ERR_TRAP(g_bms_err)

                    break;
                }

                default:
                {
                    break;
                }
            }

            break;
        }

        default:
        {
            break;
        }
    }
}
/**********************************************************************************************************************
 * End of function bms_state_machine
 *********************************************************************************************************************/


/**********************************************************************************************************************
 * Function Name: usb_event_process
 * Description  : This function processes a USB event and checks for input data from terminal.
 * Arguments    : none
 * Return Value : FSP_SUCCESS     No error was returned.
 *                FSP_ERR_...     Inherit from g_basic0.p_api routines.
 *********************************************************************************************************************/
fsp_err_t usb_event_process(void)
{
    fsp_err_t fsp_err = FSP_SUCCESS;

    /* Get the USB event */
    fsp_err = g_basic0.p_api->eventGet(&g_basic0_ctrl, &g_usb_event);
    FSP_ERROR_RETURN(FSP_SUCCESS == fsp_err, fsp_err); // Check for errors.

    /* USB event received by eventGet */
    switch(g_usb_event)
    {
        case USB_STATUS_CONFIGURED:
        {
            memset(s_usb_rx_buff, 0, sizeof(s_usb_rx_buff));

            fsp_err = g_basic0.p_api->read(&g_basic0_ctrl, s_usb_rx_buff, USB_READ_BUF_SIZE, USB_CLASS_PCDC);
            FSP_ERROR_RETURN(FSP_SUCCESS == fsp_err, fsp_err); // Check for errors.
            break;
        }

        case USB_STATUS_READ_COMPLETE:
        {
            if(g_usb_attach)
            {
                fsp_err = g_basic0.p_api->read(&g_basic0_ctrl, &s_usb_rx_buff[0], USB_READ_BUF_SIZE, USB_CLASS_PCDC);
                FSP_ERROR_RETURN(FSP_SUCCESS == fsp_err, fsp_err); // Check for errors.
            }
            else
            {
                ;
            }

            /* Process user command */
            command_handler(&s_usb_rx_buff[0]);

            break;
        }

        case USB_STATUS_REQUEST: // Receive Class Request
        {
            fsp_err = g_basic0.p_api->setupGet(&g_basic0_ctrl, &g_usb_setup);
            FSP_ERROR_RETURN(FSP_SUCCESS == fsp_err, fsp_err); // Check for errors.

            /* Check for the specific CDC class request IDs */
            if (USB_PCDC_SET_LINE_CODING == (g_usb_setup.request_type & USB_BREQUEST))
            {
                fsp_err = g_basic0.p_api->periControlDataGet(&g_basic0_ctrl,
                                                             (uint8_t *) &s_line_coding,
                                                             LINE_CODING_LENGTH );
                FSP_ERROR_RETURN(FSP_SUCCESS == fsp_err, fsp_err); // Check for errors.
            }
            else if (USB_PCDC_GET_LINE_CODING == (g_usb_setup.request_type & USB_BREQUEST))
            {
                fsp_err = g_basic0.p_api->periControlDataSet(&g_basic0_ctrl,
                                                             (uint8_t *) &s_line_coding,
                                                             LINE_CODING_LENGTH );
                FSP_ERROR_RETURN(FSP_SUCCESS == fsp_err, fsp_err); // Check for errors.
            }
            else if (USB_PCDC_SET_CONTROL_LINE_STATE == (g_usb_setup.request_type & USB_BREQUEST))
            {
                fsp_err = g_basic0.p_api->periControlStatusSet(&g_basic0_ctrl, USB_SETUP_STATUS_ACK);
                FSP_ERROR_RETURN(FSP_SUCCESS == fsp_err, fsp_err); // Check for errors.
            }
            else
            {
                ;
            }

            break;
        }

        case USB_STATUS_DETACH:
        case USB_STATUS_SUSPEND:
        {
            g_usb_attach = false;

            break;
        }

        case USB_STATUS_RESUME:
        {
            g_usb_attach = true;
            break;
        }

        default:
        {
            break;
        }
    }

    return fsp_err;
}
/**********************************************************************************************************************
 * End of function usb_event_process
 *********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: command_handler
 * Description  : This function decodes a user command and sets transition flags.
 * Arguments    : p_rx_data     Pointer to input data.
 * Return Value : none
 *********************************************************************************************************************/
static void command_handler(uint8_t * p_rx_data)
{
    fsp_err_t fsp_err = FSP_SUCCESS;

    /* Switch case evaluation of user input */
    switch(*p_rx_data)
    {
        case GET_VOLT_TEMP:
        {
            s_bms_trans_flags.goto_measure = true; // Set transition flag

            break;
        }

        case AUTOSCAN:
        {
            s_bms_trans_flags.goto_autoscan = true; // Set transition flag

            break;
        }

        case SLEEP:
        {
            s_bms_trans_flags.goto_sleep = true; // Set transition flag

            break;
        }

        case MORE_INFO:
        {
            print_to_console(g_kit_info);
            if (FSP_SUCCESS != fsp_err)
            {
                TURN_D7_ON // Turn ON RED LED to indicate fatal error
                APP_ERR_TRAP(fsp_err);
            }

            break;
        }

        case FETS_CTRL:
        {
            s_pow_fets_toggle = true;

            break;
        }

        case CARRIAGE_RETURN:
        {
            s_carriage_return = true;

            break;
        }

        default:
        {
            break;
        }
    }
}
/**********************************************************************************************************************
 * End of function command_handler
 *********************************************************************************************************************/


/**********************************************************************************************************************
 * Function Name: measured_data_process
 * Description  : Convert, assemble and send measured data to console.
 * Arguments    : p_data            Pointer to measured data structure.
 * Return Value : FSP_SUCCESS       No error was returned.
 *                FSP_ERR_...       Inherit from print_to_console()
 *********************************************************************************************************************/
static fsp_err_t measured_data_process(u_raa489220_all_meas_t * p_data)
{
    fsp_err_t err = FSP_SUCCESS;

    uint16_t buffer_index = 0x0000;

    /* Clear print data buffer before copying. */
    memset(s_usb_print_dat_buff, '\0', USB_LONG_PACKET_SIZE - 1U);

    /* Calculate current data filled length. */
    buffer_index = ((uint16_t) (strlen(s_usb_print_dat_buff)));

    /* Append the data from current buffer_index. */
    sprintf( (char *) &s_usb_print_dat_buff[buffer_index],
            "%s%d.%02d \r\n",
            MEAS_MSG_2,
            FL_INT(p_data->measurements.v_pack),
            FL_FRAC_V(p_data->measurements.v_pack));

    /* Calculate current data filled length. */
    buffer_index = ((uint16_t) (strlen(s_usb_print_dat_buff)));

    /* Append the data from current buffer_index. */
    sprintf( (char *) &s_usb_print_dat_buff[buffer_index],
            "%s%d.%03d \r\n",
            MEAS_MSG_3,
            FL_INT(p_data->measurements.i_discharge),
            FL_FRAC_I(p_data->measurements.i_discharge));

    /* Calculate current data filled length. */
    buffer_index = ((uint16_t) (strlen(s_usb_print_dat_buff)));

    /* Append the data from current buffer_index. */
    sprintf( (char *) &s_usb_print_dat_buff[buffer_index],
            "%s%d.%03d \r\n",
            MEAS_MSG_4,
            FL_INT(p_data->measurements.i_charge),
            FL_FRAC_I(p_data->measurements.i_charge));

    /* Update index count. */
    buffer_index = ((uint16_t) (strlen(s_usb_print_dat_buff)));

    /* Append the data from current buffer_index. */
    sprintf( (char *) &s_usb_print_dat_buff[buffer_index], "%s", TABLE_HEADER_CVOLTS);

    /* Update index count. */
    buffer_index = ((uint16_t) (strlen(s_usb_print_dat_buff)));

    /* Append data for every cell input of the current device. */
    for(uint16_t i = 0; i < BFE_CELL_NUM; i++ )
    {
        /* Update index count. */
        buffer_index = ((uint16_t) (strlen(s_usb_print_dat_buff)));

        /* Append the data from current buffer_index. */
        sprintf((char *) &s_usb_print_dat_buff[buffer_index], " %d.%02d |",
                FL_INT(p_data->vector[i]),
                FL_FRAC_V(p_data->vector[i]));
    }

    /* Send data to terminal. */
    err = print_to_console(s_usb_print_dat_buff);
    FSP_ERROR_RETURN(FSP_SUCCESS == err, err); // Check for errors.

    /* Clear print data buffer before copying. */
    memset(s_usb_print_dat_buff, '\0', USB_LONG_PACKET_SIZE - 1U);

    /* Update index count. */
    buffer_index = ((uint16_t) (strlen(s_usb_print_dat_buff)));

    /* Append the data from current buffer_index. */
    sprintf((char *) &s_usb_print_dat_buff[buffer_index], "\r\n %s", TABLE_HEADER_TEMPS);

    /* Update index count. */
    buffer_index = ((uint16_t) (strlen(s_usb_print_dat_buff)));

    /* Append data for every temperature input of the current device. */
    for(uint16_t i = 0; i < BFE_THERM_NUM; i++ )
    {
        /* Update index count. */
        buffer_index = ((uint16_t) (strlen(s_usb_print_dat_buff)));

        /* Append the data from current buffer_index. */
        sprintf((char *) &s_usb_print_dat_buff[buffer_index],
                "    %d.%02d    |",
                FL_INT(p_data->vector[i + 15]),
                FL_FRAC_V(p_data->vector[i + 15]));
    }

    /* Send data to terminal. */
    err = print_to_console(s_usb_print_dat_buff);
    FSP_ERROR_RETURN(FSP_SUCCESS == err, err); // Check for errors.

    /* Clear print data buffer before copying. */
    memset(s_usb_print_dat_buff, '\0', USB_LONG_PACKET_SIZE - 1U);

    return err;
}
/**********************************************************************************************************************
* End of function measured_data_process
**********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: measured_data_processs
 * Description  : Convert, assemble and send measured data to console.
 * Arguments    : p_data            Pointer to measured data structure.
 * Return Value : FSP_SUCCESS       No error was returned.
 *                FSP_ERR_...       Inherit from print_to_console()
 *********************************************************************************************************************/
static fsp_err_t measured_data_process2(u_raa489220_sys_scan_meas_t * p_data, int index)
{
    fsp_err_t err = FSP_SUCCESS;

    uint16_t buffer_index = 0x0000;

    /* Clear print data buffer before copying. */
    memset(s_usb_print_dat_buff, '\0', USB_LONG_PACKET_SIZE - 1U);

    /* Calculate current data filled length. */
    buffer_index = ((uint16_t) (strlen(s_usb_print_dat_buff)));

    /* Append the data from current buffer_index. */
    sprintf( (char *) &s_usb_print_dat_buff[buffer_index],
            "\r\n\r\n\r\n ------------ %02d -------------",
            index);

    /* Calculate current data filled length. */
    buffer_index = ((uint16_t) (strlen(s_usb_print_dat_buff)));

    /* Append the data from current buffer_index. */
    sprintf( (char *) &s_usb_print_dat_buff[buffer_index],
            "%s%d.%02d \r\n",
            MEAS_MSG_2,
            FL_INT(p_data->measurements.v_pack),
            FL_FRAC_V(p_data->measurements.v_pack));

    /* Calculate current data filled length. */
    buffer_index = ((uint16_t) (strlen(s_usb_print_dat_buff)));

    /* Append the data from current buffer_index. */
    sprintf( (char *) &s_usb_print_dat_buff[buffer_index],
            "%s%d.%02d \r\n",
            MEAS_MSG_5,
            FL_INT(p_data->measurements.v_cell_max),
            FL_FRAC_V(p_data->measurements.v_cell_max));

    /* Calculate current data filled length. */
    buffer_index = ((uint16_t) (strlen(s_usb_print_dat_buff)));

    /* Append the data from current buffer_index. */
    sprintf( (char *) &s_usb_print_dat_buff[buffer_index],
            "%s%d.%02d \r\n",
            MEAS_MSG_6,
            FL_INT(p_data->measurements.v_cell_min),
            FL_FRAC_V(p_data->measurements.v_cell_min));

    /* Calculate current data filled length. */
    buffer_index = ((uint16_t) (strlen(s_usb_print_dat_buff)));

    /* Append the data from current buffer_index. */
    sprintf( (char *) &s_usb_print_dat_buff[buffer_index],
            "%s%d.%02d \r\n",
            MEAS_MSG_3,
            FL_INT(p_data->measurements.i_discharge),
            FL_FRAC_I(p_data->measurements.i_discharge));

    /* Calculate current data filled length. */
    buffer_index = ((uint16_t) (strlen(s_usb_print_dat_buff)));

    /* Append the data from current buffer_index. */
    sprintf( (char *) &s_usb_print_dat_buff[buffer_index],
            "%s%d.%02d \r\n",
            MEAS_MSG_4,
            FL_INT(p_data->measurements.i_charge),
            FL_FRAC_I(p_data->measurements.i_charge));

    /* Send data to terminal. */
    err = print_to_console(s_usb_print_dat_buff);
    FSP_ERROR_RETURN(FSP_SUCCESS == err, err); // Check for errors.

    /* Clear print data buffer before copying. */
    memset(s_usb_print_dat_buff, '\0', USB_LONG_PACKET_SIZE - 1U);

    /* Update index count. */
    buffer_index = ((uint16_t) (strlen(s_usb_print_dat_buff)));

    /* Append the data from current buffer_index. */
    sprintf((char *) &s_usb_print_dat_buff[buffer_index], "\r\n %s", TABLE_HEADER_TEMPS);

    /* Update index count. */
    buffer_index = ((uint16_t) (strlen(s_usb_print_dat_buff)));

    /* Append data for every temperature input of the current device. */
    for(uint16_t i = 0; i < BFE_THERM_NUM; i++ )
    {
        /* Update index count. */
        buffer_index = ((uint16_t) (strlen(s_usb_print_dat_buff)));

        /* Append the data from current buffer_index. */
        sprintf((char *) &s_usb_print_dat_buff[buffer_index],
                "    %d.%02d    |",
                FL_INT(p_data->vector[i + 3]),
                FL_FRAC_V(p_data->vector[i + 3]));
    }


    /* Send data to terminal. */
    err = print_to_console(s_usb_print_dat_buff);
    FSP_ERROR_RETURN(FSP_SUCCESS == err, err); // Check for errors.

    /* Clear print data buffer before copying. */
    memset(s_usb_print_dat_buff, '\0', USB_LONG_PACKET_SIZE - 1U);

    return err;
}
/**********************************************************************************************************************
* End of function measured_data_process2
**********************************************************************************************************************/


/**********************************************************************************************************************
 * Function Name: print_to_console
 * Description  : Send a string to console via USB interface.
 * Arguments    : p_data            Pointer to data string.
 * Return Value : FSP_SUCCESS       No error was returned.
 *                FSP_ERR_...       Inherit from g_basic0.p_api
 *********************************************************************************************************************/
static fsp_err_t print_to_console(char * p_data)
{
    fsp_err_t err = FSP_SUCCESS;

    uint32_t  len = (uint32_t) strlen(p_data);

    if(g_usb_attach)
    {
        /* Send data to the USB host. */
        err = g_basic0.p_api->write(&g_basic0_ctrl, (uint8_t*) p_data, len, USB_CLASS_PCDC);
        FSP_ERROR_RETURN(FSP_SUCCESS == err, err); // Check for errors.

        /* Wait for end of transfer. */
        err = check_for_write_complete();
        FSP_ERROR_RETURN(FSP_SUCCESS == err, FSP_ERR_USB_FAILED); // Check for errors.
    }
    else
    {
        ;
    }

    return err;
}
/**********************************************************************************************************************
* End of function print_to_console
**********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: check_for_write_complete
 * Description  : Check if data was send via USB.
 * Arguments    : none
 * Return Value : FSP_SUCCESS       No error was returned.
 *                FSP_ERR_...       Inherit from g_basic0.p_api
 *********************************************************************************************************************/
static fsp_err_t check_for_write_complete(void)
{
    fsp_err_t err = FSP_SUCCESS;

    usb_status_t usb_write_event = USB_STATUS_NONE;
    int32_t      timeout_count   = UINT16_MAX;

    do
    {
        /* Obtain USB related events. */
        err = g_basic0.p_api->eventGet(&g_basic0_ctrl, &usb_write_event);
        FSP_ERROR_RETURN(FSP_SUCCESS == err, err); // Check for errors.

        timeout_count--;

        if(timeout_count <= 0)
        {
            err = (fsp_err_t) USB_STATUS_NONE;

            break;
        }
    }
    while(USB_STATUS_WRITE_COMPLETE != usb_write_event);

    return err;
}
/**********************************************************************************************************************
* End of function check_for_write_complete
**********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: bfe_fault_management
 * Description  : This function processes BFE faults. Some of can be cleared on clear true input.
 * Arguments    : in_bfe_err        Input BMS error code.
 *                clear_fault       Clear fault control input parameter
 * Return Value : BMS_SUCCESS       No error was returned.
 *                BMS_ERR_....      Copied from in_bfe_err input parameter.
 *********************************************************************************************************************/
static e_bms_err_t bfe_fault_management(e_bfe_err_t in_bfe_err, bool clear_fault)
{
    e_bms_err_t bms_err = BMS_ERR_BFE; // Error status
    e_bfe_err_t bfe_err = in_bfe_err;

    static st_raa489220_faults_t s_faults = {0};

    static bool s_success    = false;
    static bool s_do_once_fm = true;

    switch(in_bfe_err)
    {
        case BFE_SUCCESS:
        {
            if(g_bfe0_ctrl.is_fault_detected == true) // Fault pin assertion.
            {
                if(s_do_once_fm == true) // Send to terminal once
                {
                    memset(&s_faults, 0, sizeof(s_faults));

                    /* Check if the BFE has automatically entered Low Power Mode after fault detection. */
                    if(g_bfe0_ctrl.is_low_power == true)
                    {
                        /* Go to Idle Mode */
                        g_bfe_err = g_bfe0.p_api->p_modeSet(&g_bfe0_ctrl, BFE_MODE_IDLE);
                        if(bfe_err != BFE_SUCCESS)
                        {
                            /* Print banner info to console. */
                            print_to_console(g_fault_msg1);

                            /* Halt */
                            APP_ERR_TRAP(bfe_err)
                        }

                        /* Configure the Battery Front End. */
                        bfe_err = g_bfe0.p_api->p_setup(&g_bfe0_ctrl, &g_bfe0_cfg);
                        if(bfe_err != BFE_SUCCESS)
                        {
                            /* Print banner info to console. */
                            print_to_console(g_fault_msg1);

                            /* Halt */
                            APP_ERR_TRAP(bfe_err)
                        }
                    }
                    else
                    {
                        ;
                    }

                    /* Read the fault. */
                    bfe_err = g_bfe0.p_api->p_faultsAllRead(&g_bfe0_ctrl, &s_faults);
                    if(bfe_err != BFE_SUCCESS)
                    {
                        /* Print banner info to console. */
                        print_to_console(g_fault_msg1);

                        /* Halt */
                        APP_ERR_TRAP(bfe_err)
                    }

                    faults_data_process(&s_faults);

                    s_do_once_fm = false;
                }
                else
                {
                    ;
                }


                /* Manage faults that can be cleared with a command. */
                if((s_faults.flt_charge_over_current == true)
                   || (s_faults.flt_discharge_over_current == true)
                   || (s_faults.flt_short_circuit == true)
                   || (s_faults.flt_overvoltage == true)
                   || (s_faults.flt_undervoltage == true)
                   || (s_faults.flt_over_temeprature == true)
                   || (s_faults.flt_under_temeprature == true)
                   )
                {
                    if(clear_fault == true)
                    {
                        /* Clear all faults. */
                        bfe_err = g_bfe0.p_api->p_faultsAllClear(&g_bfe0_ctrl, &s_success);
                        if(bfe_err != BFE_SUCCESS)
                        {
                            /* Print banner info to console. */
                            print_to_console(g_fault_msg1);

                            /* Halt */
                            APP_ERR_TRAP(bfe_err)
                        }

                        if(s_success == true)
                        {
                            s_do_once_fm = true;

                            bms_err = BMS_SUCCESS; //Exit Fault mode

                            /* Print message to console. */
                            print_to_console(g_fault_msg3);
                        }
                        else
                        {
                            ;
                        }

                    }
                    else
                    {
                        ;
                    }
                }
                else
                {
                    ;
                }

                /* Manage faults that require BFE reset. */
                if((s_faults.flt_delta_cell_ov == true)
                   || (s_faults.flt_internal_over_temp == true)
                   || (s_faults.flt_self_test == true)
                   || (s_faults.flt_regulator == true)
                   || (s_faults.flt_vtemp == true)
                   || (s_faults.flt_open_wire == true)
                   || (s_faults.flt_comm_timeout == true))
                {
                    if(clear_fault == true)
                    {
                        /* Restart the BFE. */
                        bfe_err = g_bfe0.p_api->p_reset(&g_bfe0_ctrl, BFE_RESET_TYPE_HARD);
                        if(bfe_err != BFE_SUCCESS)
                        {
                            /* Print banner info to console. */
                            print_to_console(g_fault_msg1);

                            /* Halt */
                            APP_ERR_TRAP(bfe_err)
                        }

                        /* Check if registry defaults are correctly loaded. */
                        bfe_err = g_bfe0.p_api->p_memoryCheck(&g_bfe0_ctrl, BFE_CHECK_DEF_VALS);
                        if(bfe_err != BFE_SUCCESS)
                        {
                            /* Print banner info to console. */
                            print_to_console(g_fault_msg1);

                            /* Halt */
                            APP_ERR_TRAP(bfe_err)
                        }

                        /* Configure the Battery Front End. */
                        bfe_err = g_bfe0.p_api->p_setup(&g_bfe0_ctrl, &g_bfe0_cfg);
                        if(bfe_err != BFE_SUCCESS)
                        {
                            /* Print banner info to console. */
                            print_to_console(g_fault_msg1);

                            /* Halt */
                            APP_ERR_TRAP(bfe_err)
                        }

                        s_do_once_fm = true;

                        bms_err = BMS_SUCCESS; //Exit Fault mode

                        /* Print message to console. */
                        print_to_console(g_fault_msg3);
                    }
                    else
                    {
                        ;
                    }
                }
                else
                {
                    ;
                }
            }
            else
            {
                bms_err = BMS_SUCCESS; //Exit Fault mode
            }

            break;
        }

        case BFE_ERR_COMM_FAULT:
        case BFE_ERR_CRC_INCORRECT:
        case BFE_ERR_WRITE_VERIFY:
        {
            /* Process communication failure, incorrect CRC or NAC response. */
            if(s_do_once_fm == true) // Send to terminal once
            {
                /* Print banner info to console. */
                print_to_console(g_fault_msg2);

                s_do_once_fm = false;
            }
            else
            {
                ;
            }

            if(clear_fault == true)
            {
                /* Restart the BFE. */
                bfe_err = g_bfe0.p_api->p_reset(&g_bfe0_ctrl, BFE_RESET_TYPE_HARD);
                if(bfe_err != BFE_SUCCESS)
                {
                    /* Print banner info to console. */
                    print_to_console(g_fault_msg1);

                    /* Halt */
                    APP_ERR_TRAP(bfe_err)
                }

                /* Check if registry defaults are correctly loaded. */
                bfe_err = g_bfe0.p_api->p_memoryCheck(&g_bfe0_ctrl, BFE_CHECK_DEF_VALS);
                if(bfe_err != BFE_SUCCESS)
                {
                    /* Print banner info to console. */
                    print_to_console(g_fault_msg1);

                    /* Halt */
                    APP_ERR_TRAP(bfe_err)
                }

                /* Configure the Battery Front End. */
                bfe_err = g_bfe0.p_api->p_setup(&g_bfe0_ctrl, &g_bfe0_cfg);
                if(bfe_err != BFE_SUCCESS)
                {
                    /* Print banner info to console. */
                    print_to_console(g_fault_msg1);

                    /* Halt */
                    APP_ERR_TRAP(bfe_err)
                }

                s_do_once_fm = true;

                bms_err = BMS_SUCCESS; //Exit Fault mode

                /* Print message to console. */
                print_to_console(g_fault_msg3);
            }
            else
            {
                ;
            }

            break;
        }


        default:
        {
            if(clear_fault == true)
            {
                /* Print banner info to console. */
                print_to_console(g_fault_msg1);
            }
            else
            {
                ;
            }

            break;
        }
    }

    return bms_err;
}
/**********************************************************************************************************************
 * End of function fault_management
 *********************************************************************************************************************/


/**********************************************************************************************************************
 * Function Name: faults_data_process
 * Description  : Convert, assemble and send faults data to console.
 * Arguments    : p_data            Pointer to measured data structure.
 * Return Value : FSP_SUCCESS       No error was returned.
 *                FSP_ERR_...       Inherit from print_to_console()
 *********************************************************************************************************************/
static fsp_err_t faults_data_process(st_raa489220_faults_t * p_faults)
{
    fsp_err_t err = FSP_SUCCESS;

    uint16_t  buffer_index = 0x0000;

    /* Clear kit info buffer before updating data. */
    memset(s_usb_print_dat_buff, '\0', USB_LONG_PACKET_SIZE - 1U);

    /* Charge overcurrent fault processing. */
    if(p_faults->flt_charge_over_current == true)
    {
        /* Update index count. */
        buffer_index = ((uint16_t)(strlen(s_usb_print_dat_buff)));

        /* Append the data from current buffer_index. */
        sprintf((char *)&s_usb_print_dat_buff[buffer_index], "\r\n Charge overcurrent!");
    }
    else
    {
        ;
    }

    /* Discharge overcurrent fault. */
    if(p_faults->flt_discharge_over_current == true)
    {
        /* Update index count. */
        buffer_index = ((uint16_t)(strlen(s_usb_print_dat_buff)));

        /* Append the data from current buffer_index. */
        sprintf((char *)&s_usb_print_dat_buff[buffer_index], "\r\n Discharge overcurrent!");
    }
    else
    {
        ;
    }

    /* Short-circuit fault. */
    if(p_faults->flt_short_circuit == true)
    {
        /* Update index count. */
        buffer_index = ((uint16_t)(strlen(s_usb_print_dat_buff)));

        /* Append the data from current buffer_index. */
        sprintf((char *)&s_usb_print_dat_buff[buffer_index], "\r\n Short-circuit!");
    }
    else
    {
        ;
    }

    /* Overvoltage fault. */
    if(p_faults->flt_overvoltage == true)
    {
        /* Update index count. */
        buffer_index = ((uint16_t)(strlen(s_usb_print_dat_buff)));

        if(p_faults->flt_lock_out == true)
        {
            /* Append the data from current buffer_index. */
            sprintf((char *)&s_usb_print_dat_buff[buffer_index], "\r\n Overvoltage lockout fault!");
        }
        else
        {
            /* Append the data from current buffer_index. */
            sprintf((char *)&s_usb_print_dat_buff[buffer_index], "\r\n Overvoltage fault!");
        }
    }
    else
    {
        ;
    }

    /* Undervoltage fault. */
    if(p_faults->flt_undervoltage == true)
    {
        /* Update index count. */
        buffer_index = ((uint16_t)(strlen(s_usb_print_dat_buff)));

        if(p_faults->flt_lock_out == true)
        {
            /* Append the data from current buffer_index. */
            sprintf((char *)&s_usb_print_dat_buff[buffer_index], "\r\n Undervoltage lockout fault!");
        }
        else
        {
            /* Append the data from current buffer_index. */
            sprintf((char *)&s_usb_print_dat_buff[buffer_index], "\r\n Undervoltage fault!");
        }
    }
    else
    {
        ;
    }

    /* Delta cell overvoltage fault. */
    if(p_faults->flt_delta_cell_ov == true)
    {
        /* Update index count. */
        buffer_index = ((uint16_t)(strlen(s_usb_print_dat_buff)));

        /* Append the data from current buffer_index. */
        sprintf((char *)&s_usb_print_dat_buff[buffer_index], "\r\n Delta cell overvoltage fault!");
    }
    else
    {
        ;
    }

    /* Over-temperature fault. */
    if(p_faults->flt_over_temeprature == true)
    {
        /* Update index count. */
        buffer_index = ((uint16_t)(strlen(s_usb_print_dat_buff)));

        /* Append the data from current buffer_index. */
        sprintf((char *)&s_usb_print_dat_buff[buffer_index], "\r\n Over-temperature fault!");
    }
    else
    {
        ;
    }

    /* Under-temperature fault. */
    if(p_faults->flt_under_temeprature == true)
    {
        /* Update index count. */
        buffer_index = ((uint16_t)(strlen(s_usb_print_dat_buff)));

        /* Append the data from current buffer_index. */
        sprintf((char *)&s_usb_print_dat_buff[buffer_index], "\r\n Under-temperature fault!");
    }
    else
    {
        ;
    }

    /* Internal over-temperature fault. */
    if(p_faults->flt_internal_over_temp == true)
    {
        /* Update index count. */
        buffer_index = ((uint16_t)(strlen(s_usb_print_dat_buff)));

        /* Append the data from current buffer_index. */
        sprintf((char *)&s_usb_print_dat_buff[buffer_index], "\r\n Internal over-temperature fault!");
    }
    else
    {
        ;
    }

    /* Communication timeout fault. */
    if(p_faults->flt_comm_timeout == true)
    {
        /* Update index count. */
        buffer_index = ((uint16_t)(strlen(s_usb_print_dat_buff)));

        /* Append the data from current buffer_index. */
        sprintf((char *)&s_usb_print_dat_buff[buffer_index], "\r\n Communication timeout!");
    }
    else
    {
        ;
    }

    /* Self-test fault. */
    if(p_faults->flt_self_test == true)
    {
        /* Update index count. */
        buffer_index = ((uint16_t)(strlen(s_usb_print_dat_buff)));

        /* Append the data from current buffer_index. */
        sprintf((char *)&s_usb_print_dat_buff[buffer_index], "\r\n Self-test failure!");
    }
    else
    {
        ;
    }

    /* Internal regulator fault. */
    if(p_faults->flt_regulator == true)
    {
        /* Update index count. */
        buffer_index = ((uint16_t)(strlen(s_usb_print_dat_buff)));

        /* Append the data from current buffer_index. */
        sprintf((char *)&s_usb_print_dat_buff[buffer_index], "\r\n Internal regulator failure!");
    }
    else
    {
        ;
    }

    /* Open-wire on thermistor inputs. */
    if(p_faults->flt_vtemp == true)
    {
        /* Update index count. */
        buffer_index = ((uint16_t)(strlen(s_usb_print_dat_buff)));

        /* Append the data from current buffer_index. */
        sprintf((char *)&s_usb_print_dat_buff[buffer_index], "\r\n Open-wire on thermistor inputs!");
    }
    else
    {
        ;
    }

    /* Open-wire on cell inputs. */
    if(p_faults->flt_open_wire == true)
    {
        /* Update index count. */
        buffer_index = ((uint16_t)(strlen(s_usb_print_dat_buff)));

        /* Append the data from current buffer_index. */
        sprintf((char *)&s_usb_print_dat_buff[buffer_index], "\r\n Open-wire on VCn inputs!");
    }
    else
    {
        ;
    }

    /* Update index count. */
    buffer_index = ((uint16_t) (strlen(s_usb_print_dat_buff)));

    /* Append the data from current buffer_index. */
    sprintf((char *)&s_usb_print_dat_buff[buffer_index], "\r\n\r\n Press 'Enter' to clear all faults..");

    /* Print acquired data to console. */
    err = print_to_console(s_usb_print_dat_buff);
    FSP_ERROR_RETURN(FSP_SUCCESS == err, err); // Check for errors.

    return err;
}
/**********************************************************************************************************************
* End of function faults_data_process
**********************************************************************************************************************/


/**********************************************************************************************************************
 * Function Name: periodic_timer_callback
 * Description  : AGT callback function.
 * Arguments    : p_args            Pointer to callback arguments.
 * Return Value : none
 *********************************************************************************************************************/
void periodic_timer_callback (timer_callback_args_t * p_args)
{
    if(TIMER_EVENT_CYCLE_END == p_args->event)
    {
        s_periodic_timer_event_flag = true;
    }
}
/**********************************************************************************************************************
 * End of function periodic_timer_callback
 *********************************************************************************************************************/
